@@ -247,7 +247,7 @@ proc mkAuthor {} {
   set w .author.frame
 
   label $w.msg0   -text "Author and Maintener of xlock"
-  label $w.msg1   -text "Maintained by: David A. Bagley (bagleyd@bigfoot.com)"
+  label $w.msg1   -text "Maintained by: David A. Bagley (bagleyd@tux.org)"
   label $w.msg2   -text "Original Author: Patrick J. Naughton (naughton@eng.sun.com)"
   label $w.msg3   -text "Mailstop 21-14 Sun Microsystems Laboratories,"
   label $w.msg4   -text "Inc.  Mountain View, CA  94043 15//336-1080"
