@@ -5,7 +5,7 @@
 # Add the menu file with load resources 
 # and the exit button 
 # Add the load_process procedure loading 
-# the ressource file ~/XLock 
+# the resource file ~/XLock 
 #
 
 #function find in demo: mkStyles.tcl
@@ -247,7 +247,7 @@ proc mkAuthor {} {
   set w .author.frame
 
   label $w.msg0   -text "Author and Maintener of xlock"
-  label $w.msg1   -text "Maintained by: David A. Bagley (bagleyd@bigfoot.com)"
+  label $w.msg1   -text "Maintained by: David A. Bagley (bagleyd@tux.org)"
   label $w.msg2   -text "Original Author: Patrick J. Naughton (naughton@eng.sun.com)"
   label $w.msg3   -text "Mailstop 21-14 Sun Microsystems Laboratories,"
   label $w.msg4   -text "Inc.  Mountain View, CA  94043 15//336-1080"
@@ -647,6 +647,7 @@ coral\
 crystal\
 daisy\
 dclock\
+decay\
 deco\
 demon\
 dilemma\
@@ -668,9 +669,11 @@ hyper\
 ico\
 ifs\
 image\
+invert\
 julia\
 kaleid\
 kumppa\
+lament\
 laser\
 life\
 life1d\
@@ -679,8 +682,10 @@ lightning\
 lisa\
 lissie\
 loop\
+lyapunov\
 mandelbrot\
 marquee\
+matrix\
 maze\
 moebius\
 morph3d\
