@@ -90,6 +90,7 @@ lst.addItem("coral");
 lst.addItem("crystal");
 lst.addItem("daisy");
 lst.addItem("dclock");
+lst.addItem("decay");
 lst.addItem("deco");
 lst.addItem("demon");
 lst.addItem("dilemma");
@@ -111,9 +112,11 @@ lst.addItem("hyper");
 lst.addItem("ico");
 lst.addItem("ifs");
 lst.addItem("image");
+lst.addItem("invert");
 lst.addItem("julia");
 lst.addItem("kaleid");
 lst.addItem("kumppa");
+lst.addItem("lament");
 lst.addItem("laser");
 lst.addItem("life");
 lst.addItem("life1d");
@@ -121,8 +124,10 @@ lst.addItem("life3d");
 lst.addItem("lisa");
 lst.addItem("lissie");
 lst.addItem("loop");
-lst.addItem("marquee");
+lst.addItem("lyapunov");
 lst.addItem("mandelbrot");
+lst.addItem("marquee");
+lst.addItem("matrix");
 lst.addItem("maze");
 lst.addItem("moebius");
 lst.addItem("morph3d");
