@@ -78,32 +78,39 @@ static const char sccsid[] = "@(#)ant.c	4.11 98/06/18 xlockmore";
  * neighbors of 0 randomizes it between 3, 4 and 6.
  * 8, 9 12 are available also but not recommended.
  */
-extern int  neighbors;
 
+#define DEF_NEIGHBORS  "0"      /* choose random value */
 #define DEF_TRUCHET  "False"
+#define DEF_EYES  "False"
 #define DEF_SHARPTURN  "False"
 
+static int  neighbors;
 static Bool truchet;
+static Bool eyes;
 static Bool sharpturn;
 
 static XrmOptionDescRec opts[] =
 {
+	{"-neighbors", ".ant.neighbors", XrmoptionSepArg, (caddr_t) NULL},
 	{"-truchet", ".ant.truchet", XrmoptionNoArg, (caddr_t) "on"},
 	{"+truchet", ".ant.truchet", XrmoptionNoArg, (caddr_t) "off"},
+	{"-eyes", ".ant.eyes", XrmoptionNoArg, (caddr_t) "on"},
+	{"+eyes", ".ant.eyes", XrmoptionNoArg, (caddr_t) "off"},
 	{"-sharpturn", ".ant.sharpturn", XrmoptionNoArg, (caddr_t) "on"},
 	{"+sharpturn", ".ant.sharpturn", XrmoptionNoArg, (caddr_t) "off"}
 };
 static argtype vars[] =
 {
+	{(caddr_t *) & neighbors, "neighbors", "Neighbors", DEF_NEIGHBORS, t_Int},
 	{(caddr_t *) & truchet, "truchet", "Truchet", DEF_TRUCHET, t_Bool},
+	{(caddr_t *) & eyes, "eyes", "Eyes", DEF_EYES, t_Bool},
    {(caddr_t *) & sharpturn, "sharpturn", "SharpTurn", DEF_SHARPTURN, t_Bool}
 };
 static OptionStruct desc[] =
 {
-#if 0
-      {"-neighbors num", "squares 4 or 8, hexagons 6, triangles 3, 9 or 12"},
-#endif
+	{"-neighbors num", "squares 4 or 8, hexagons 6, triangles 3, 9 or 12"},
 	{"-/+truchet", "turn on/off Truchet lines"},
+	{"-/+eyes", "turn on/off eyes"},
 	{"-/+sharpturn", "turn on/off sharp turns (6 or 12 neighbors only)"}
 };
 
@@ -156,6 +163,7 @@ typedef struct {
 	int         n;
 	int         redrawing, redrawpos;
 	int         truchet;	/* Only for Turk modes */
+	int         eyes;
 	int         sharpturn;	/* Only for even neighbors > 4 (i.e. 6 and 12) */
 	statestruct machine[NUMSTIPPLES * STATES];
 	unsigned char *tape;
@@ -473,8 +481,8 @@ fillcell(ModeInfo * mi, GC gc, int col, int row)
 		ap->shape.hexagon[0].x = ap->xb + ccol * ap->xs;
 		ap->shape.hexagon[0].y = ap->yb + crow * ap->ys;
 		if (ap->xs == 1 && ap->ys == 1)
-			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-			ap->shape.hexagon[0].x, ap->shape.hexagon[0].y, 1, 1);
+			XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+			ap->shape.hexagon[0].x, ap->shape.hexagon[0].y);
 		else
 			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
 			    ap->shape.hexagon, 6, Convex, CoordModePrevious);
@@ -488,9 +496,9 @@ fillcell(ModeInfo * mi, GC gc, int col, int row)
 		ap->shape.triangle[orient][0].x = ap->xb + col * ap->xs;
 		ap->shape.triangle[orient][0].y = ap->yb + row * ap->ys;
 		if (ap->xs <= 3 || ap->ys <= 3)
-			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+			XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
 			((orient) ? -1 : 1) + ap->shape.triangle[orient][0].x,
-				       ap->shape.triangle[orient][0].y, 1, 1);
+				       ap->shape.triangle[orient][0].y);
 		else {
 			if (orient)
 				ap->shape.triangle[orient][0].x += (ap->xs / 2 - 1);
@@ -644,48 +652,113 @@ drawtruchet(ModeInfo * mi, int col, int row,
 }
 
 static void
-draw_anant(ModeInfo * mi, int col, int row)
+draw_anant(ModeInfo * mi, int direction, int col, int row)
 {
-	XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WHITE_PIXEL(mi));
+	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+
+	XSetForeground(display, MI_GC(mi), MI_WHITE_PIXEL(mi));
 	fillcell(mi, MI_GC(mi), col, row);
-#if 0				/* Can not see eyes */
-	{
-		antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
-		Display    *display = MI_DISPLAY(mi);
-		Window      window = MI_WINDOW(mi);
+	if (ap->eyes && ap->xs > 3 && ap->ys > 3) {		/* Draw Eyes */
 
-		if (ap->xs > 2 && ap->ys > 2) {		/* Draw Eyes */
+		XSetForeground(display, MI_GC(mi), MI_BLACK_PIXEL(mi));
+#if 0  /* under construction */
+		if (ap->neighbors == 6) {
+			int         ccol = 2 * col + !(row & 1), crow = 2 * row;
+			int         side;
+			XPoint      hex;
 
-			XSetForeground(display, MI_GC(mi), MI_BLACK_PIXEL(mi));
+			hex.x = ap->xb + ccol * ap->xs;
+			hex.y = ap->yb + crow * ap->ys;
+			for (side = 0; side < direction / 60; side++) {
+				hex.x += ap->shape.hexagon[side].x;
+				hex.y += ap->shape.hexagon[side].y;
+			}
+		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+		hex.x, hex.y, 7, 7);
+
+					/*case 0:
+					case 60:
+					case 120:
+					case 180:
+					case 240:
+					case 300:
+					default:
+						(void) fprintf(stderr, "wrong direction %d\n", direction); */
+		} else
+#endif
+		if (ap->neighbors == 4 || ap->neighbors == 8) {
 			switch (direction) {
 				case 0:
 					XDrawPoint(display, window, MI_GC(mi),
-					    ap->xb + ap->xs - 1, ap->yb + 1);
+						ap->xb + ap->xs * (col + 1) - 3,
+					 	ap->yb + ap->ys * row + ap->ys / 2 - 2);
 					XDrawPoint(display, window, MI_GC(mi),
-						   ap->xb + ap->xs - 1, ap->yb + ap->ys - 2);
+						ap->xb + ap->xs * (col + 1) - 3,
+					 	ap->yb + ap->ys * row + ap->ys / 2);
+					break;
+				case 45:
+					XDrawPoint(display, window, MI_GC(mi),
+					 	ap->xb + ap->xs * (col  + 1) - 4,
+						ap->yb + ap->ys * row + 1);
+					XDrawPoint(display, window, MI_GC(mi),
+					 	ap->xb + ap->xs * (col  + 1) - 3,
+						ap->yb + ap->ys * row + 2);
+					break;
+				case 90:
+					XDrawPoint(display, window, MI_GC(mi),
+					 	ap->xb + ap->xs * col + ap->xs / 2 - 2,
+						ap->yb + ap->ys * row + 1);
+					XDrawPoint(display, window, MI_GC(mi),
+					 	ap->xb + ap->xs * col + ap->xs / 2,
+						ap->yb + ap->ys * row + 1);
+					break;
+				case 135:
+					XDrawPoint(display, window, MI_GC(mi),
+					 	ap->xb + ap->xs * col + 2,
+						ap->yb + ap->ys * row + 1);
+					XDrawPoint(display, window, MI_GC(mi),
+					 	ap->xb + ap->xs * col + 1,
+						ap->yb + ap->ys * row + 2);
 					break;
 				case 180:
-					XDrawPoint(display, window, MI_GC(mi), ap->xb, ap->yb + 1);
-					XDrawPoint(display, window, MI_GC(mi), ap->xb, ap->yb + ap->ys - 2);
+					XDrawPoint(display, window, MI_GC(mi),
+						ap->xb + ap->xs * col + 1,
+					 	ap->yb + ap->ys * row + ap->ys / 2 - 2);
+					XDrawPoint(display, window, MI_GC(mi),
+						ap->xb + ap->xs * col + 1,
+					 	ap->yb + ap->ys * row + ap->ys / 2);
+					break;
+				case 225:
+					XDrawPoint(display, window, MI_GC(mi),
+					 	ap->xb + ap->xs * col + 2,
+						ap->yb + ap->ys * (row + 1) - 3);
+					XDrawPoint(display, window, MI_GC(mi),
+					 	ap->xb + ap->xs * col + 1,
+						ap->yb + ap->ys * (row + 1) - 4);
 					break;
-					if (neighbors == 4) {
-				case 90:
-						XDrawPoint(display, window, MI_GC(mi), ap->xb + 1, ap->yb);
-						XDrawPoint(display, window, MI_GC(mi),
-						ap->xb + ap->xs - 2, ap->yb);
-						break;
 				case 270:
-						XDrawPoint(display, window, MI_GC(mi),
-							   ap->xb + 1, ap->yb + ap->ys - 1);
-						XDrawPoint(display, window, MI_GC(mi),
-							   ap->xb + ap->xs - 2, ap->yb + ap->ys - 1);
-						break;
-					}	/* else BEE */
+					XDrawPoint(display, window, MI_GC(mi),
+					 	ap->xb + ap->xs * col + ap->xs / 2 - 2,
+						ap->yb + ap->ys * (row + 1) - 3);
+					XDrawPoint(display, window, MI_GC(mi),
+					 	ap->xb + ap->xs * col + ap->xs / 2,
+						ap->yb + ap->ys * (row + 1) - 3);
+					break;
+				case 315:
+					XDrawPoint(display, window, MI_GC(mi),
+					 	ap->xb + ap->xs * (col + 1) - 4,
+						ap->yb + ap->ys * (row + 1) - 3);
+					XDrawPoint(display, window, MI_GC(mi),
+					 	ap->xb + ap->xs * (col + 1) - 3,
+						ap->yb + ap->ys * (row + 1) - 4);
+					break;
 				default:
+					(void) fprintf(stderr, "wrong direction %d\n", direction);
 			}
 		}
 	}
-#endif
 }
 
 #if 0
@@ -993,11 +1066,14 @@ init_ant(ModeInfo * mi)
 
 	if (MI_IS_FULLRANDOM(mi)) {
 		ap->truchet = (Bool) (LRAND() & 1);
+		ap->eyes = (Bool) (LRAND() & 1);
 		ap->sharpturn = (Bool) (LRAND() & 1);
 	} else {
 		ap->truchet = truchet;
+		ap->eyes = eyes;
 		ap->sharpturn = sharpturn;
 	}
+	ap->eyes = (ap->eyes && ap->xs > 3 && ap->ys > 3);
 	/* Exclude odd # of neighbors, stepping forward not defined */
 	if (!NRAND(NUMSTIPPLES) && ((ap->neighbors + 1) % 2)) {
 		getTable(mi, (int) (NRAND(NTABLES)));
@@ -1029,7 +1105,7 @@ init_ant(ModeInfo * mi)
 		ap->ants[i].direction = dir;
 		ap->ants[i].state = 0;
 	}
-	draw_anant(mi, col, row);
+	draw_anant(mi, dir, col, row);
 }
 
 void
@@ -1091,7 +1167,7 @@ draw_ant(ModeInfo * mi)
 		/* If edge than wrap it */
 		old_dir = ((status->direction < ANGLES) ? anant->direction : old_dir);
 		position_of_neighbor(ap, old_dir, &(anant->col), &(anant->row));
-		draw_anant(mi, anant->col, anant->row);
+		draw_anant(mi, anant->direction, anant->col, anant->row);
 	}
 	if (++ap->generation > MI_CYCLES(mi)) {
 		init_ant(mi);
