@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 4 -*- */
-/* dclock --- floating digital clock */
+/* dclock --- floating digital clock or message */
 
 #if !defined( lint ) && !defined( SABER )
 static const char sccsid[] = "@(#)dclock.c	4.07 97/11/24 xlockmore";
@@ -21,19 +21,21 @@ static const char sccsid[] = "@(#)dclock.c	4.07 97/11/24 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
+ * 07-May-99: New "savers" added for y2k and second millennium countdowns.
+ *            Tom Schmidt <tschmidt@micron.com>
  * 04-Dec-98: New "savers" added for hiv, veg, and lab.
-              hiv one due to Kenneth Stailey <kstailey@disclosure.com>
+ *            hiv one due to Kenneth Stailey <kstailey@disclosure.com>
  * 10-Aug-98: Population Explosion and Tropical Forest Countdown stuff
  *            I tried to get precise numbers but they may be off a few percent.
  *            Whether or not, its still pretty scary IMHO. 
  *            Although I am a US citizen... I have the default for area in
- *            metric.  David Bagley <bagleyd@bigfoot.com>
+ *            metric.  David Bagley <bagleyd@tux.org>
  * 10-May-97: Compatible with xscreensaver
  * 29-Aug-95: Written.
  */
 
 /*-
- *  Some of my calcualations laid bare...  (I have a little problem with
+ *  Some of my calculations laid bare...  (I have a little problem with
  *  the consistency of the numbers I got at the Bronx Zoo but proportions
  *  were figured to be 160.70344 people a minute increase not 180 and
  *  35.303144 hectares (87.198766 acres) a minute decrease not 247 (100 acres).
@@ -103,11 +105,19 @@ static const char sccsid[] = "@(#)dclock.c	4.07 97/11/24 xlockmore";
 #define METRIC 1
 #endif
 #if METRIC
-#define AREA_STRING "Hectares"
+#ifdef NL
+# define AREA_STRING "Hectare"
+#else
+# define AREA_STRING "Hectares"
+#endif
 #define AREA_MIN 35.303144
 #define AREA_TIME_START 1184193000.0
 #else
-#define AREA_STRING "Acres"
+#ifdef NL
+# define AREA_STRING "Acre"
+#else
+# define AREA_STRING "Acres"
+#endif
 #define AREA_MIN 87.198766
 #define AREA_TIME_START 2924959000.0
 #endif
@@ -119,18 +129,25 @@ static const char sccsid[] = "@(#)dclock.c	4.07 97/11/24 xlockmore";
 #define LAB_TIME_START 0
 #define VEG_MIN 9506.426344209
 #define VEG_TIME_START 0
+/* epoch time at midnight 1 January 2000 UTC */
+#define Y2K_TIME_START (((30 * 365) + 7) * 24 * 60 * 60)
+#define Y2001_TIME_START (Y2K_TIME_START + 366 * 24 * 60 * 60)
 
 #define DEF_POPEX  "False"
 #define DEF_FOREST "False"
 #define DEF_HIV    "False"
 #define DEF_LAB    "False"
 #define DEF_VEG    "False"
+#define DEF_Y2K    "False"
+#define DEF_Y2001  "False"
 
 static Bool popex;
 static Bool forest;
 static Bool hiv;
 static Bool lab;
 static Bool veg;
+static Bool y2k;
+static Bool millennium;
 
 static XrmOptionDescRec opts[] =
 {
@@ -143,7 +160,11 @@ static XrmOptionDescRec opts[] =
 	{"-lab", ".dclock.lab", XrmoptionNoArg, (caddr_t) "on"},
 	{"+lab", ".dclock.lab", XrmoptionNoArg, (caddr_t) "off"},
 	{"-veg", ".dclock.veg", XrmoptionNoArg, (caddr_t) "on"},
-	{"+veg", ".dclock.veg", XrmoptionNoArg, (caddr_t) "off"}
+	{"+veg", ".dclock.veg", XrmoptionNoArg, (caddr_t) "off"},
+	{"-y2k", ".dclock.y2k", XrmoptionNoArg, (caddr_t) "on"},
+	{"+y2k", ".dclock.y2k", XrmoptionNoArg, (caddr_t) "off"},
+	{"-millennium", ".dclock.millennium", XrmoptionNoArg, (caddr_t) "on"},
+	{"+millennium", ".dclock.millennium", XrmoptionNoArg, (caddr_t) "off"}
 };
 static argtype vars[] =
 {
@@ -151,15 +172,19 @@ static argtype vars[] =
 	{(caddr_t *) & forest, "forest", "Forest", DEF_FOREST, t_Bool},
 	{(caddr_t *) & hiv, "hiv", "Hiv", DEF_HIV, t_Bool},
 	{(caddr_t *) & lab, "lab", "Lab", DEF_LAB, t_Bool},
-	{(caddr_t *) & veg, "veg", "Veg", DEF_VEG, t_Bool}
+	{(caddr_t *) & veg, "veg", "Veg", DEF_VEG, t_Bool},
+	{(caddr_t *) & y2k, "y2k", "Y2K", DEF_Y2K, t_Bool},
+	{(caddr_t *) & millennium, "millennium", "Millennium", DEF_Y2001, t_Bool}
 };
 static OptionStruct desc[] =
 {
-	{"-/+popex", "turn on/off population explosion"},
-	{"-/+forest", "turn on/off tropical forest descruction"},
+	{"-/+popex", "turn on/off population explosion counter"},
+	{"-/+forest", "turn on/off tropical forest destruction counter"},
 	{"-/+hiv", "turn on/off HIV infection counter"},
 	{"-/+lab", "turn on/off Animal Research counter"},
-	{"-/+veg", "turn on/off Animal Consumation counter"}
+	{"-/+veg", "turn on/off Animal Consumation counter"},
+	{"-/+y2k", "turn on/off Year 2000 countdown"},
+	{"-/+millennium", "turn on/off 3rd Millennium (1 January 2001) countdown"}
 };
 
 ModeSpecOpt dclock_opts =
@@ -170,22 +195,37 @@ ModStruct   dclock_description =
 {"dclock", "init_dclock", "draw_dclock", "release_dclock",
  "refresh_dclock", "init_dclock", NULL, &dclock_opts,
  10000, 1, 10000, 1, 0.3, 64, "",
- "Shows a floating digital clock", 0, NULL};
+ "Shows a floating digital clock or message", 0, NULL};
 
 #endif
 
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
 #include <time.h>
 
 #ifdef FR
-#define POPEX_STRING "Population Mondiale"
-#define PEOPLE_STRING " Personnes"
-#define FOREST_STRING "Nombre des Forets Tropicales"
-#define TROPICAL_STRING " Zones Tropicales en "
-#define HIV_STRING "Infection par le HIV a l'heure actuelle dans le monde est de"
-#define CASES_STRING ""
-#define LAB_STRING "Used in research"
-#define VEG_STRING "Consumed for Food by Mankind"
-#define YEAR_STRING " animals this year"
+#define POPEX_STRING "Population mondiale"
+#define PEOPLE_STRING " personnes"
+#define FOREST_STRING "Taille des forts tropicales"
+#define TROPICAL_STRING " zones tropicales en "
+#define HIV_STRING "Infections par le SIDA dans le monde"
+#define CASES_STRING " cas"
+#define LAB_STRING "Animaux utiliss dans la recherche"
+#define VEG_STRING "Animaux consomms comme aliments par l'homme"
+#define YEAR_STRING " cette anne"
+#define Y2K_STRING "Dcompte pour l'an 2000 (1er Janvier 2000)"
+#define POST_Y2K_STRING "Temps depuis l'an 2000 (1er Janvier 2000)"
+#define Y2001_STRING "Dcompte pour le Second Millnaire (1er Janvier 2001)"
+#define POST_Y2001_STRING "Temps depuis le Second Millnaire (1er Janvier 2001)"
+#define DAY "jour"
+#define DAYS "jours"
+#define HOUR "heure"
+#define HOURS "heures"
+#define MINUTE "minute"
+#define MINUTES "minutes"
+#define SECOND "seconde"
+#define SECONDS "secondes"
 #else
 #ifdef NL
 #define POPEX_STRING "Wereld populatie"
@@ -194,9 +234,24 @@ ModStruct   dclock_description =
 #define TROPICAL_STRING " Tropisch gebied in "
 #define HIV_STRING "Huidige staat HIV infecties wereldwijd"
 #define CASES_STRING " gevallen"
-#define LAB_STRING "Used in research"
-#define VEG_STRING "Consumed for Food by Mankind"
-#define YEAR_STRING " animals this year"
+#define LAB_STRING "Verbruikt tijdens onderzoek"
+#define VEG_STRING "Opgegeten door de mens"
+#define YEAR_STRING " dieren dit jaar"
+#define Y2K_STRING "Aftellen tot Y2K (1 Januari 2000, 0.00 uur)"
+#define POST_Y2K_STRING "Tijd sinds Y2K (1 Januari 2000)"
+#define Y2001_STRING "Aftellen tot het einde van het tweede Millennium (1 Januari 2001, 0.00 uur)"
+#define POST_Y2001_STRING "Verstreken tijd sinds het begin van het derde Millennium (1 Januari 2001)"
+
+#define Y2001_STRING "Aftellen tot het tweede millennium (1 Januari 2001)"
+#define POST_Y2001_STRING "Tijd sinds het tweede millennium (1 Januari 2001)"
+#define DAY "dag"
+#define DAYS "dagen"
+#define HOUR "uur"
+#define HOURS "uren"
+#define MINUTE "minuut"
+#define MINUTES "minuten"
+#define SECOND "seconde"
+#define SECONDS "seconden"
 #else
 #define POPEX_STRING "World Population"
 #define PEOPLE_STRING " People"
@@ -207,6 +262,18 @@ ModStruct   dclock_description =
 #define LAB_STRING "Used in research"
 #define VEG_STRING "Consumed for Food by Mankind"
 #define YEAR_STRING " animals this year"
+#define Y2K_STRING "Countdown to Y2K (1 January 2000, 0:00 hour)"
+#define POST_Y2K_STRING "Time since Y2K (1 January 2000)"
+#define Y2001_STRING "Countdown to the end of the Second Millennium (1 January 2001, 0:00 hour)"
+#define POST_Y2001_STRING "Time since the start of the Third Millennium (1 January 2001)"
+#define DAY "day"
+#define DAYS "days"
+#define HOUR "hour"
+#define HOURS "hours"
+#define MINUTE "minute"
+#define MINUTES "minutes"
+#define SECOND "second"
+#define SECONDS "seconds"
 #endif
 #endif
 
@@ -220,6 +287,7 @@ typedef struct {
 	char       *str, str1[40], str2[40], str1old[80], str2old[80];
 	char       *str1pta, *str2pta, *str1ptb, *str2ptb;
 	time_t      timenew, timeold;
+	int         tzoffset;
 	short       maxx, maxy, clockx, clocky;
 	short       text_height, text_width, text_width1, text_width2;
 	short       text_start1, text_start2;
@@ -229,7 +297,7 @@ typedef struct {
 	int         pixw, pixh;
 	Pixmap      pixmap;
 	GC          fgGC, bgGC;
-	Bool        popex, forest, hiv, lab, veg;
+	Bool        popex, forest, hiv, lab, veg, y2k, millennium;
 } dclockstruct;
 
 static dclockstruct *dclocks = NULL;
@@ -280,6 +348,47 @@ convert(double x, char *string)
 	string[k++] = '\0';
 }
 
+static void
+dayhrminsec(long timeCount, int tzoffset, char *string)
+{
+	int	 days, hours, minutes, secs;
+	char	*buf;
+
+	timeCount = abs(timeCount);
+	days = timeCount / 86400;
+	hours = (timeCount / 3600) % 24;
+	minutes = (timeCount / 60) % 60;
+	secs = timeCount % 60;
+	buf = (char *) malloc(16);
+	(void) sprintf(string, "%d ", days);
+	if (days == 1)
+		(void) strcat(string, DAY);
+	else
+		(void) strcat(string, DAYS);
+	(void) sprintf(buf, ", %d ", hours);
+	(void) strcat(string, buf);
+	if (hours == 1)
+		(void) strcat(string, HOUR);
+	else
+		(void) strcat(string, HOURS);
+	(void) sprintf(buf, ", %d ", minutes);
+	(void) strcat(string, buf);
+	if (minutes == 1)
+		(void) strcat(string, MINUTE);
+	else
+		(void) strcat(string, MINUTES);
+	(void) sprintf(buf, ", %d ", secs);
+	(void) strcat(string, buf);
+	if (secs == 1)
+		(void) strcat(string, SECOND);
+	else
+		(void) strcat(string, SECONDS);
+	if (!tzoffset)
+		(void) strcat(string, " (UTC)");
+	(void) free((void *) buf);
+	buf = NULL;
+}
+
 static void
 drawDclock(ModeInfo * mi)
 {
@@ -290,42 +399,15 @@ drawDclock(ModeInfo * mi)
 	short       xold, yold;
 	char       *tmppt;
 
-	xold = dp->clockx;
-	yold = dp->clocky;
-	dp->clockx += dp->dx;
-	dp->clocky += dp->dy;
-
-	if (dp->maxx < dp->text_start1) {
-		if (dp->clockx < dp->maxx + dp->text_start1 ||
-		    dp->clockx > dp->text_start1) {
-			dp->dx = -dp->dx;
-			dp->clockx += dp->dx;
-		}
-	} else if (dp->maxx > dp->text_start1) {
-		if (dp->clockx > dp->maxx + dp->text_start1 ||
-		    dp->clockx < dp->text_start1) {
-			dp->dx = -dp->dx;
-			dp->clockx += dp->dx;
-		}
-	}
-	if (dp->maxy < mode_font->ascent) {
-		if (dp->clocky > mode_font->ascent || dp->clocky < dp->maxy) {
-			dp->dy = -dp->dy;
-			dp->clocky += dp->dy;
-		}
-	} else if (dp->maxy > mode_font->ascent) {
-		if (dp->clocky > dp->maxy || dp->clocky < mode_font->ascent) {
-			dp->dy = -dp->dy;
-			dp->clocky += dp->dy;
-		}
-	}
-	if (!dp->popex && !dp->forest && !dp->hiv && !dp->lab && !dp->veg) {
+	if (!dp->popex && !dp->forest && !dp->hiv && !dp->lab && !dp->veg &&
+	    !dp->y2k && !dp->millennium) {
 		if (dp->timeold != (dp->timenew = time((time_t *) NULL))) {
 			/* only parse if time has changed */
 			dp->timeold = dp->timenew;
 			dp->str = ctime(&dp->timeold);
 
-			if (!dp->popex && !dp->forest && !dp->hiv && !dp->lab && !dp->veg) {
+			if (!dp->popex && !dp->forest && !dp->hiv && !dp->lab &&
+			    !dp->veg && !dp->y2k && !dp->millennium) {
 
 				/* keep last disp time so it can be cleared even if it changed */
 				tmppt = dp->str1ptb;
@@ -360,7 +442,9 @@ drawDclock(ModeInfo * mi)
 			(void) strncpy(dp->str2pta + 11, (dp->str + 20), 4);
 		}
 	} else {
-		unsigned long timeNow = seconds();
+		unsigned long timeNow, timeLocal;
+		timeNow = seconds();
+		timeLocal = timeNow - dp->tzoffset;
 
 		if (dp->popex) {
 			convert(PEOPLE_TIME_START + (PEOPLE_MIN / 60.0) * timeNow, dp->str2);
@@ -390,6 +474,68 @@ drawDclock(ModeInfo * mi)
 			(void) strcat(dp->str2, YEAR_STRING);
 			dp->str2pta = dp->str2;
 			dp->str2ptb = dp->str2pta;
+		} else if (dp->y2k) {
+			if (Y2K_TIME_START >= timeLocal)
+				dp->str1pta = Y2K_STRING;
+			else
+				dp->str1pta = POST_Y2K_STRING;
+			dp->str1ptb = dp->str1pta;
+			dayhrminsec(Y2K_TIME_START - timeLocal, dp->tzoffset, dp->str2);
+			dp->str2pta = dp->str2;
+			dp->str2ptb = dp->str2pta;
+		} else if (dp->millennium) {
+			if (Y2001_TIME_START >= timeLocal)
+				dp->str1pta = Y2001_STRING;
+			else
+				dp->str1pta = POST_Y2001_STRING;
+			dp->str1ptb = dp->str1pta;
+			dayhrminsec(Y2001_TIME_START - timeLocal, dp->tzoffset, dp->str2);
+			dp->str2pta = dp->str2;
+			dp->str2ptb = dp->str2pta;
+		}
+	}
+	/* Recalculate string width since it can change */
+	xold = dp->clockx;
+	yold = dp->clocky;
+	dp->text_width1 = XTextWidth(mode_font, dp->str1pta, strlen(dp->str1pta));
+	dp->text_width2 = XTextWidth(mode_font, dp->str2pta, strlen(dp->str2pta));
+	if (dp->text_width1 > dp->text_width2) {
+		dp->text_width = dp->text_width1;
+		dp->text_start1 = 0;
+		dp->text_start2 = (dp->text_width - dp->text_width2) / 2;
+	} else {
+		dp->text_width = dp->text_width2;
+		dp->text_start1 = (dp->text_width - dp->text_width1) / 2;
+		dp->text_start2 = 0;
+	}
+	dp->width = MI_WIDTH(mi);
+	dp->height = MI_HEIGHT(mi);
+	dp->maxx = dp->width - dp->text_width;
+	dp->maxy = dp->height - dp->text_height - mode_font->descent;
+	dp->clockx += dp->dx;
+	dp->clocky += dp->dy;
+	if (dp->maxx < dp->text_start1) {
+		if (dp->clockx < dp->maxx + dp->text_start1 ||
+		    dp->clockx > dp->text_start1) {
+			dp->dx = -dp->dx;
+			dp->clockx += dp->dx;
+		}
+	} else if (dp->maxx > dp->text_start1) {
+		if (dp->clockx >= dp->maxx + dp->text_start1 ||
+		    dp->clockx <= dp->text_start1) {
+			dp->dx = -dp->dx;
+			dp->clockx += dp->dx;
+		}
+	}
+	if (dp->maxy < mode_font->ascent) {
+		if (dp->clocky > mode_font->ascent || dp->clocky < dp->maxy) {
+			dp->dy = -dp->dy;
+			dp->clocky += dp->dy;
+		}
+	} else if (dp->maxy > mode_font->ascent) {
+		if (dp->clocky > dp->maxy || dp->clocky < mode_font->ascent) {
+			dp->dy = -dp->dy;
+			dp->clocky += dp->dy;
 		}
 	}
 	if (dp->pixw != dp->text_width || dp->pixh != 2 * dp->text_height) {
@@ -401,7 +547,7 @@ drawDclock(ModeInfo * mi)
 			XFreeGC(display, dp->bgGC);
 		if (dp->pixmap) {
 			XFreePixmap(display, dp->pixmap);
-			MI_CLEARWINDOW(mi);
+			MI_CLEARWINDOWCOLORMAPFAST(mi, gc, MI_BLACK_PIXEL(mi));
 		}
 		dp->pixw = dp->text_width;
 		dp->pixh = 2 * dp->text_height;
@@ -448,7 +594,10 @@ init_dclock(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
 	dclockstruct *dp;
-	unsigned long timeNow;
+	unsigned long timeNow, timeLocal;
+#if defined(HAVE_TZSET) && !(defined(BSD) && BSD >= 199306)
+	extern long timezone;
+#endif
 
 	if (dclocks == NULL) {
 		logbase = log(BASE);
@@ -468,8 +617,10 @@ init_dclock(ModeInfo * mi)
 	dp->hiv = False;
 	dp->lab = False;
 	dp->veg = False;
+	dp->y2k = False;
+	dp->millennium = False;
 	if (MI_IS_FULLRANDOM(mi)) {
-		switch (NRAND(6)) {
+		switch (NRAND(8)) {
 			case 1:
 				dp->popex = True;
 				break;
@@ -485,6 +636,12 @@ init_dclock(ModeInfo * mi)
 			case 5:
 				dp->veg = True;
 				break;
+			case 6:
+				dp->y2k = True;
+				break;
+			case 7:
+				dp->millennium = True;
+				break;
 			default:
 				break;
 		}
@@ -494,6 +651,8 @@ init_dclock(ModeInfo * mi)
 		dp->hiv = hiv;
 		dp->lab = lab;
 		dp->veg = veg;
+		dp->y2k = y2k;
+		dp->millennium = millennium;
 	}
 
 	if (mode_font == None)
@@ -504,11 +663,26 @@ init_dclock(ModeInfo * mi)
 			XSetFont(display, MI_GC(mi), mode_font->fid);
 	}
 	/* (void)time(&dp->timenew); */
+#if defined(HAVE_TZSET) && (!defined(HAVE_TIMELOCAL) || (defined(BSD) && BSD >= 199306))
+	(void) tzset();
+#endif
 	dp->timeold = dp->timenew = time((time_t *) NULL);
+#if defined(HAVE_TIMELOCAL) && !(defined(BSD) && BSD >= 199306)
+	if (!dp->tzoffset)
+		dp->tzoffset = timelocal(&dp->timeold) - timegm(&dp->timeold);
+#else
+#ifdef HAVE_TZSET
+	dp->tzoffset = (int)timezone;
+#endif
+#endif
+	if (dp->tzoffset > 86400 || dp->tzoffset < -86400)
+		dp->tzoffset = 0;
 	dp->str = ctime(&dp->timeold);
 	dp->dx = (LRAND() & 1) ? 1 : -1;
 	dp->dy = (LRAND() & 1) ? 1 : -1;
 
+	timeNow = seconds();
+	timeLocal = timeNow - dp->tzoffset;
 	if (dp->popex) {
 		dp->str1pta = POPEX_STRING;
 		dp->str1ptb = dp->str1pta;
@@ -524,6 +698,18 @@ init_dclock(ModeInfo * mi)
 	} else if (dp->veg) {
 		dp->str1pta = VEG_STRING;
 		dp->str1ptb = dp->str1pta;
+	} else if (dp->y2k) {
+		if (Y2K_TIME_START >= timeLocal)
+			dp->str1pta = Y2K_STRING;
+		else
+			dp->str1pta = POST_Y2K_STRING;
+		dp->str1ptb = dp->str1pta;
+	} else if (dp->millennium) {
+		if (Y2001_TIME_START >= timeLocal)
+			dp->str1pta = Y2001_STRING;
+		else
+			dp->str1pta = POST_Y2001_STRING;
+		dp->str1ptb = dp->str1pta;
 	} else {
 		(void) strncpy(dp->str1, (dp->str + 11), 8);
 		dp->hour = (short) (dp->str1[0] - 48) * 10 + (short) (dp->str1[1] - 48);
@@ -550,7 +736,6 @@ init_dclock(ModeInfo * mi)
 		dp->str1pta = dp->str1;
 		dp->str1ptb = dp->str1old;
 	}
-	timeNow = seconds();
 	if (dp->popex) {
 		convert(PEOPLE_TIME_START + (PEOPLE_MIN / 60.0) * timeNow, dp->str2);
 		(void) strcat(dp->str2, PEOPLE_STRING);
@@ -578,6 +763,12 @@ init_dclock(ModeInfo * mi)
 			dp->str2);
 		(void) strcat(dp->str2, YEAR_STRING);
 		dp->str2pta = dp->str2;
+	} else if (dp->y2k) {
+		dayhrminsec(Y2K_TIME_START - timeLocal, dp->tzoffset, dp->str2);
+		dp->str2pta = dp->str2;
+	} else if (dp->millennium) {
+		dayhrminsec(Y2001_TIME_START - timeLocal, dp->tzoffset, dp->str2);
+		dp->str2pta = dp->str2;
 	} else {
 		dp->str2pta = dp->str2;
 		dp->str2ptb = dp->str2old;
