@@ -47,7 +47,7 @@ static const char sccsid[] = "@(#)eyes.c	4.07 97/11/24 xlockmore";
  "*cycles: 5 \n" \
  "*ncolors: 200 \n" \
  "*bitmap: \n" \
- "*mouse: False \n"
+ "*trackmouse: False \n"
 #include "xlockmore.h"		/* in xscreensaver distribution */
 
 #else /* STANDALONE */
@@ -58,8 +58,28 @@ static const char sccsid[] = "@(#)eyes.c	4.07 97/11/24 xlockmore";
 
 #ifdef MODE_eyes
 
+#define DEF_TRACKMOUSE  "False"
+
+static Bool trackmouse;
+
+static XrmOptionDescRec opts[] =
+{
+	{"-track", ".eyes.trackmouse", XrmoptionNoArg, (caddr_t) "on"},
+	{"+track", ".eyes.trackmouse", XrmoptionNoArg, (caddr_t) "off"}
+};
+
+static argtype vars[] =
+{
+	{(caddr_t *) & trackmouse, "trackmouse", "TrackMouse", DEF_TRACKMOUSE, t_Bool}
+};
+
+static OptionStruct desc[] =
+{
+	{"-/+trackmouse", "turn on/off the tracking of the mouse"}
+};
+
 ModeSpecOpt eyes_opts =
-{0, NULL, 0, NULL, NULL};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
 
 #ifdef USE_MODULES
 ModStruct   eyes_description =
@@ -547,7 +567,7 @@ move_fly(ModeInfo * mi, Fly * f)
 	int         left = (f->side == FLY_SIDE_LEFT) ? -(f->width) : 0;
 	int         right = (f->side == FLY_SIDE_RIGHT) ? win_width :
 	win_width - f->width;
-	Bool        track_p = MI_IS_MOUSE(mi);
+	Bool        track_p = trackmouse;
 	int         cx, cy;
 
 
@@ -935,7 +955,7 @@ init_eyes(ModeInfo * mi)
 
 	init_fly(mi, &(ep->fly));	/* init the bouncer */
 
-	if (MI_IS_MOUSE(mi) && !ep->cursor) {	/* Create an invisible cursor */
+	if (trackmouse && !ep->cursor) {	/* Create an invisible cursor */
 		Pixmap      bit;
 		XColor      black;
 
