@@ -2310,13 +2310,18 @@ gl_init(ModeInfo * mi)
 	if (do_texture) {
 #ifdef HAVE_GLBINDTEXTURE
 		int         i;
-#ifdef MESA
+#if ((MESA_MAJOR_VERSION < 3 ) || (( MESA_MAJOR_VERSION == 3 ) && (MESA_MINOR_VERSION == 0 )))
 		/* E.Lassauge - 11/23/98
 		 * It looks like there's a bug in MESA (up to 3.1beta1) for the
 		 * "Default" texture (named '0'). For this texture
 		 * I added a glBindTexture and the same glTexParameteri
 		 * as for the specific textures. Now it does'nt core
 		 * anymore.
+		 * 
+		 * 22-mar-99 for latest version of Mesa this codes wipes
+		 * out the structure on the sides of the cube. Leaving the code
+		 * out did not crash the mode on my VMS system.
+	 	 *      Jouk
 		 */
 		glBindTexture(GL_TEXTURE_2D, 0);
 		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, interior_color);
