@@ -716,8 +716,9 @@ release_kumppa(ModeInfo * mi)
 				}
 				if (s->cgc)
 					XFreeGC(display, s->cgc);
-				if (s->cmap)
+				if (s->cmap) {
 					XFreeColormap(display, s->cmap);
+				}
 			}
 			if (s->acosinus)
 				(void) free((void *) s->acosinus);
