@@ -840,6 +840,12 @@ refresh_life1d(ModeInfo * mi)
 	life1dstruct *lp = &life1ds[MI_SCREEN(mi)];
 	int         row, col, nrow;
 
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+	/* This is needed when another program changes the colormap. */
+	free_stuff(MI_DISPLAY(mi), lp);
+	init_stuff(mi);
+#endif
+
 	for (row = 0; row < lp->nrows; row++) {
 		nrow = row * lp->ncols;
 		for (col = 0; col < lp->ncols; col++)
