@@ -84,10 +84,28 @@ static const char sccsid[] = "@(#)loop.c	4.13 98/10/18 xlockmore";
 /*-
  * neighbors of 0 randomizes between 4 and 6.
  */
-extern int  neighbors;
+#define DEF_NEIGHBORS  "0"      /* choose random value */
+
+static int  neighbors;
+
+static XrmOptionDescRec opts[] =
+{
+	{"-neighbors", ".loop.neighbors", XrmoptionSepArg, (caddr_t) NULL}
+};
+
+static argtype vars[] =
+{
+	{(caddr_t *) & neighbors, "neighbors", "Neighbors", DEF_NEIGHBORS, t_Int}
+};
+
+static OptionStruct desc[] =
+{
+	{"-neighbors num", "squares 4 or hexagons 6"}
+};
 
 ModeSpecOpt loop_opts =
-{0, NULL, 0, NULL, NULL};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
 
 #ifdef USE_MODULES
 ModStruct   loop_description =
@@ -158,6 +176,7 @@ typedef struct {
 	int         mincol, minrow, maxcol, maxrow;
 	int         width, height;
 	int         redrawing, redrawpos;
+	Bool        dead;
 	unsigned char *newcells, *oldcells;
 	int         ncells[COLORS];
 	CellList   *cellList[COLORS];
@@ -752,8 +771,8 @@ fillcell(ModeInfo * mi, GC gc, int col, int row)
 		lp->shape.hexagon[0].x = lp->xb + ccol * lp->xs;
 		lp->shape.hexagon[0].y = lp->yb + crow * lp->ys;
 		if (lp->xs == 1 && lp->ys == 1)
-			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-				lp->shape.hexagon[0].x, lp->shape.hexagon[0].y, 1, 1);
+			XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+				lp->shape.hexagon[0].x, lp->shape.hexagon[0].y);
 		else
 			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
 				lp->shape.hexagon, 6, Convex, CoordModePrevious);
@@ -864,8 +883,8 @@ draw_state(ModeInfo * mi, int state)
 			lp->shape.hexagon[0].x = lp->xb + ccol * lp->xs;
 			lp->shape.hexagon[0].y = lp->yb + crow * lp->ys;
 			if (lp->xs == 1 && lp->ys == 1)
-				XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-					lp->shape.hexagon[0].x, lp->shape.hexagon[0].y, 1, 1);
+				XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+					lp->shape.hexagon[0].x, lp->shape.hexagon[0].y);
 			else
 				XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
 				  	lp->shape.hexagon, 6, Convex, CoordModePrevious);
@@ -1367,6 +1386,7 @@ draw_loop(ModeInfo * mi)
 		return;
 	}
 	MI_IS_DRAWN(mi) = True;
+	lp->dead = True;
 
 	for (j = lp->minrow; j <= lp->maxrow; j++) {
 		for (i = lp->mincol; i <= lp->maxcol; i++) {
@@ -1374,6 +1394,7 @@ draw_loop(ModeInfo * mi)
 			z = lp->oldcells + offset;
 			znew = lp->newcells + offset;
 			if (*z != *znew) {
+				lp->dead = False;
 				*z = *znew;
 				addtolist(mi, i - lp->bx, j - lp->by, *znew);
 				life = 1;
@@ -1390,7 +1411,7 @@ draw_loop(ModeInfo * mi)
 	}
 	for (i = 0; i < COLORS; i++)
 		draw_state(mi, i);
-	if (++lp->generation > MI_CYCLES(mi) /* || !life */) {
+	if (++lp->generation > MI_CYCLES(mi) || lp->dead) {
 		init_loop(mi);
 		return;
 	} else
