@@ -34,6 +34,8 @@ static const char sccsid[] = "@(#)mandelbrot.c 4.13 98/12/21 xlockmore";
  * Pickover.  These would make nice additions to add.
  *
  * Revision History:
+ * 24-Mar-99: DEM and Binary decomp options added by Tim.Auckland@Sun.COM
+ *            Ideas from Peitgen & Saupe's "The Science of Fractal Images"
  * 20-Oct-97: Written by Dan Stromberg <strombrg@nis.acs.uci.edu>
  * 17-Nov-98: Many Changes by Stromberg, including selection of
  *            interesting subregions, more extreme color ranges,
@@ -69,6 +71,8 @@ static const char sccsid[] = "@(#)mandelbrot.c 4.13 98/12/21 xlockmore";
 
 #define MINPOWER 2
 #define DEF_INCREMENT  "1.00"
+#define DEF_BINARY     "False"
+#define DEF_DEM        "False"
 /* 4.0 is best for seeing if a point is inside the set, 13 is better if
 ** you want to get a pretty corona
 */
@@ -80,19 +84,29 @@ static const char sccsid[] = "@(#)mandelbrot.c 4.13 98/12/21 xlockmore";
 	   but it's a somewhat interesting function anyway
 	 */
 static float increment;
+static Bool  binary;
+static Bool  dem;
 
 static XrmOptionDescRec opts[] =
 {
-     {"-increment", ".mandelbrot.increment", XrmoptionSepArg, (caddr_t) NULL}
+     {"-increment", ".mandelbrot.increment", XrmoptionSepArg, (caddr_t) NULL},
+     {"-binary", ".mandelbrot.binary", XrmoptionNoArg, (caddr_t) "on"},
+     {"+binary", ".mandelbrot.binary", XrmoptionNoArg, (caddr_t) "off"},
+     {"-dem", ".mandelbrot.dem", XrmoptionNoArg, (caddr_t) "on"},
+     {"+dem", ".mandelbrot.dem", XrmoptionNoArg, (caddr_t) "off"}
 };
 
 static argtype vars[] =
 {
  {(caddr_t *) & increment, "increment", "Increment", DEF_INCREMENT, t_Float},
+ {(caddr_t *) & binary, "binary", "Binary", DEF_BINARY, t_Bool},
+ {(caddr_t *) & dem, "dem", "Dem", DEF_DEM, t_Bool},
 };
 static OptionStruct desc[] =
 {
-	{"-increment value", "increasing orders"}
+	{"-increment value", "increasing orders"},
+	{"-/+binary", "turn on/off Binary Decomposition colour modulation"},
+	{"-/+dem", "turn on/off Distance Estimator Method (instead of escape time)"}
 };
 
 ModeSpecOpt mandelbrot_opts =
@@ -188,29 +202,68 @@ typedef struct {
 	int         screen_width;
 	int			screen_height;
 	int			reptop;
+	Bool		dem;
+	Bool		binary;
 } mandelstruct;
 
 static mandelstruct *mandels = NULL;
 
+/* do the iterations
+ * if binary is true, check halfplane of last iteration.
+ * if demrange is non zero, estimate lower bound of dist(c, M)
+ * Loosely based on  Peitgen & Saupe's "The Science of Fractal Images" 
+ */
 static int
-reps(complex c, double p, int r)
+reps(complex c, double p, int r, Bool binary, double demrange)
 {
 	int         rep;
 	int         escaped = 0;
 	complex     t;
+	int escape = (demrange == 0) ? ESCAPE :
+		ESCAPE*ESCAPE*ESCAPE*ESCAPE; /* 2 more iterations */
+	complex     t1;
+	complex     dt;
 
 	t = c;
+	dt.real = 1; dt.imag = 0;
 	for (rep = 0; rep < r; rep++) {
+	    t1 = t;
 		ipow(&t, p);
 		add(&t, c);
-		if (t.real * t.real + t.imag * t.imag >= ESCAPE) {
+		if (t.real * t.real + t.imag * t.imag >= escape) {
 			escaped = 1;
 			break;
 		}
+		if (demrange){
+			/* compute dt/dc 
+			 *               p-1
+			 * dt    =  p * t  * dt + 1
+			 *   k+1         k     k
+			 */
+			dt.real *= p; dt.imag *= p;
+			if(p > 2) ipow(&t1, p - 1);
+			mult(&dt, t1);
+			dt.real += 1;
+			if (dt.real * dt.real + dt.imag * dt.imag >= 1e300) {
+				escaped = 2;
+				break;
+			}
+		}
 	}
-	if (escaped)
+	if (escaped) {
+		if(demrange) {
+			double mt = sqrt(t1.real * t1.real + t1.imag * t1.imag);
+			 /* distance estimate */
+			double dist = 0.5 * mt * log(mt) /
+				sqrt(dt.real * dt.real + dt.imag * dt.imag);
+			rep = (int) 1 + 10*r*dist/demrange; /* scale for viewing */
+			if(rep > r-1) rep = r-1; /* chop into color range */
+		} else {
+			if(binary && t.imag > 0)
+				rep = (r + rep / 2) % r; /* binary decomp */
+		}
 		return rep;
-	else
+	} else
 		return r;
 }
 
@@ -268,7 +321,7 @@ Select(
 					temp.real = selected_ul->real + 
 						(selected_ul->real - selected_lr->real) * 
 						(((double)column)/sample_step);
-					r = reps(temp,power,top);
+					r = reps(temp,power,top,0,0);
 					/* Here, we just want to see if the point is in the set,
 					** not if we can make something pretty
 					*/
@@ -328,6 +381,17 @@ init_mandelbrot(ModeInfo * mi)
 	mp->screen_width = MI_WIDTH(mi);
 	mp->screen_height = MI_HEIGHT(mi);
 
+	if (MI_IS_FULLRANDOM(mi)) {
+	  switch(NRAND(3)){
+	  case 0: mp->binary = 1; mp->dem = 0; break;
+	  case 1: mp->binary = 0; mp->dem = 1; break;
+	  default:mp->binary = 0; mp->dem = 0; break;
+	  }
+	} else {
+	  mp->binary = binary;
+	  mp->dem = dem;
+	}
+
 	mp->reptop = 300;
 
 	/* these could be tuned a little closer, but the selection
@@ -408,6 +472,7 @@ draw_mandelbrot(ModeInfo * mi)
 	double      log_top;
 #endif
 
+	double      demrange;
 	unsigned int i;
 	int         j, k;
 
@@ -509,6 +574,8 @@ draw_mandelbrot(ModeInfo * mi)
 		mp->counter++;
 		return;
 	}
+	/* demrange is used to give some idea of scale */
+	demrange = mp->dem ? fabs(mp->ul.real - mp->lr.real) / 2 : 0;
 	for (h = 0; h < mp->screen_height; h++) {
 		unsigned int color;
 		int         result;
@@ -519,7 +586,7 @@ draw_mandelbrot(ModeInfo * mi)
 			(mp->ul.real-mp->lr.real)*(((double)(mp->column))/mp->screen_width);
 		c.imag = mp->ul.imag + 
 			(mp->ul.imag - mp->lr.imag)*(((double) h) / mp->screen_height);
-		result = reps(c, mp->power, mp->reptop);
+		result = reps(c, mp->power, mp->reptop, mp->binary, demrange);
 		if (result == 0 || result == mp->reptop)
 			XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 		else {
