@@ -68,12 +68,9 @@ ModStruct   puzzle_description =
 
 #endif
 
-#include <time.h>
-
 #define PUZZLE_WIDTH   image_width
 #define PUZZLE_HEIGHT    image_height
 #define PUZZLE_BITS    image_bits
-#define NOWAY 255
 #include "puzzle.xbm"
 
 #if defined( USE_XPM ) || defined( USE_XPMINC )
@@ -82,6 +79,8 @@ ModStruct   puzzle_description =
 #define DEFAULT_XPM 1
 #endif
 
+#define NOWAY 255
+
   /*int storedmoves, *moves, *position, space;  To keep track of puzzle */
 typedef struct {
 	Bool        painted;	/* For debouncing */
@@ -449,10 +448,14 @@ init_puzzle(ModeInfo * mi)
 	}
 	pp = &puzzs[MI_SCREEN(mi)];
 
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+        /* This is needed when another program changes the colormap. */
+        free_stuff(MI_DISPLAY(mi), pp);
+#endif
+	init_stuff(mi);
 	if (pp->painted && pp->windowsize.x == MI_WIDTH(mi) &&
 	    pp->windowsize.y == MI_HEIGHT(mi))
 		return;		/* Debounce since refresh_puzzle is init_puzzle */
-	init_stuff(mi);
 
 	pp->excount = MI_COUNT(mi);
 	if (pp->excount < 0) {
