@@ -32,9 +32,9 @@ static const char sccsid[] = "@(#)star.c	4.07 97/11/24 xlockmore";
  * 8-May-96: Blue on left instead of green for 3d.  It seems more common
  *           than green.  Use "-left3d Green" if you have the other kind.
  * 17-Jan-96: 3D mode for star thanks to <theiling@coli.uni-sb.de>.
- *            Get out your 3D glasses, Red on right and Blue on left.
+ *            Get out your 3D glasses, Red on left and Blue on right.
  * 14-Apr-95: Jeremie PETIT <petit@aurora.unice.fr> added a "move" feature.
- * 2-Sep-93: xlock version David Bagley <bagleyd@bigfoot.com>
+ * 2-Sep-93: xlock version David Bagley <bagleyd@tux.org>
  * 1992:     xscreensaver version Jamie Zawinski <jwz@jwz.org>
  */
 
@@ -141,7 +141,7 @@ static BitmapType trekie[] =
 };
 
 /*-
- * For 3d effect get some 3D glasses, right lens red, and left lens blue.
+ * For 3d effect get some 3D glasses, left lens red, and right lens blue.
  * Too bad monitors do not emit polarized light.
  */
 
@@ -152,7 +152,7 @@ static BitmapType trekie[] =
 #define MAXSIZE 200		/* how big (in pixels) stars are at depth 1 */
 #define DEPTH_SCALE 100		/* how many ticks there are between depths */
 #define RESOLUTION 1000
-#define MAX_DEP 0.3		/* how far the displacement can be (percents) */
+#define MAX_DEP 1.0		/* how far the displacement can be (percents) */
 #define DIRECTION_CHANGE_RATE 60
 #define MAX_DEP_SPEED 5		/* Maximum speed for movement */
 #define MOVE_STYLE 0		/* Only 0 and 1. Distinguishes the fact that
@@ -271,6 +271,8 @@ star_tick(ModeInfo * mi, astar * astars, int d)
 			astars->theta += RESOLUTION;
 		if (astars->depth < (MIN_DEPTH * DEPTH_SCALE))
 			astars->depth = 0;
+		else if (astars->depth > (MAX_DEPTH * DEPTH_SCALE))
+			astars->depth = MAX_DEPTH * DEPTH_SCALE;
 		else {
 			star_compute(mi, astars);
 			star_draw(mi, astars, True);
@@ -641,8 +643,6 @@ init_star(ModeInfo * mi)
 		sp->nstars = NRAND(-sp->nstars - MIN_STARS + 1) + MIN_STARS;
 	} else if (sp->nstars < MIN_STARS)
 		sp->nstars = MIN_STARS;
-	if (sp->speed < 1)
-		sp->speed = 1;
 	if (sp->speed > 100)
 		sp->speed = 100;
 
@@ -695,6 +695,12 @@ draw_star(ModeInfo * mi)
 	}
 	tick_stars(mi, sp->current_delta);
 	draw_trek(mi);
+#if 0
+	/* this is for making stars go backwards, not ready for prime-time */
+	if (!NRAND(500)) {
+		sp->speed = -sp->speed;
+	}
+#endif
 }
 
 void
