@@ -163,9 +163,10 @@ make_starfish(ModeInfo * mi)
 	sp->rota = 0.0004 + 0.0002 * (double) LRAND() / (double) MAXRAND;
 
 
-	if (!(LRAND() % 20))
-		sp->size *= (int) (0.35 * ((double) LRAND() / (double) MAXRAND +
-				 (double) LRAND() / (double) MAXRAND) + 0.3);
+	if (NRAND(20) == 5)
+		sp->size = (int) (sp->size *
+			(0.35 * ((double) LRAND() / (double) MAXRAND +
+				 (double) LRAND() / (double) MAXRAND) + 0.3));
 
 	{
 		static char skips[] =
