@@ -84,17 +84,17 @@ static Bool cycle_p;
 
 static XrmOptionDescRec opts[] =
 {
-	{"-tshift", ".tetris.tshift", XrmoptionNoArg, (caddr_t) "on"},
-	{"+tshift", ".tetris.tshift", XrmoptionNoArg, (caddr_t) "off"}
+	{"-cycle", ".tetris.cycle", XrmoptionNoArg, (caddr_t) "on"},
+	{"+cycle", ".tetris.cycle", XrmoptionNoArg, (caddr_t) "off"}
 };
 
 static argtype vars[] =
 {
-	{(caddr_t *) & cycle_p, "tshift", "TShift", DEF_CYCLE, t_Bool}
+	{(caddr_t *) & cycle_p, "cycle", "Cycle", DEF_CYCLE, t_Bool}
 };
 static OptionStruct desc[] =
 {
-	{"-/+tshift", "turn on/off colour cycling"}
+	{"-/+cycle", "turn on/off colour cycling"}
 };
 
 ModeSpecOpt tetris_opts =
@@ -191,11 +191,11 @@ typedef struct {
 } fieldstruct;
 
 typedef struct {
-	Bool	painted;
+	Bool	 painted;
 	int	 object, orient;
-	GC	  gc;
-	Colormap    cmap;
-	Bool	cycle_p, mono_p, no_colors;
+	GC	 gc;
+	Colormap cmap;
+	Bool	 cycle_p, mono_p, no_colors;
 	unsigned long blackpixel, whitepixel, fg, bg;
 	int	 direction;
 	unsigned long colour;
