@@ -134,7 +134,10 @@ init_xjack(ModeInfo * mi)
 	jp->rows = (MI_HEIGHT(mi) - jp->vspace - jp->vspace) / jp->line_height;
 
 	jp->left = 0xFF & (NRAND((jp->columns / 2) + 1));
-	jp->right = jp->left + (0xFF & (NRAND(jp->columns - jp->left - 10) + 10));
+	if ( jp->columns - jp->left != 10 )
+		jp->right = jp->left + (0xFF & (NRAND(jp->columns - jp->left - 10) + 10));
+	else
+		jp->right = jp->left + 10;
 	jp->x = 0;
 	jp->y = 0;
 	jp->sentences = 0;
