@@ -11,7 +11,9 @@ static char *LockProcs[] =
 "bounce",
 "braid",
 "bubble",
+#if defined( USE_GL ) && defined( HAVE_CXX )
 "bubble3d",
+#endif
 "bug",
 #ifdef USE_GL
 "cage",
@@ -24,6 +26,7 @@ static char *LockProcs[] =
 "crystal",
 "daisy",
 "dclock",
+"decay",
 "deco",
 "demon",
 "dilemma",
@@ -47,9 +50,13 @@ static char *LockProcs[] =
 "ico",
 "ifs",
 "image",
+"invert",
 "julia",
 "kaleid",
 "kumppa",
+#if defined( USE_GL ) && ( defined( USE_XPM ) || defined( USE_XPMINC ))
+"lament",
+#endif
 "laser",
 "life",
 "life1d",
@@ -58,8 +65,10 @@ static char *LockProcs[] =
 "lisa",
 "lissie",
 "loop",
+"lyapunov",
 "mandelbrot",
 "marquee",
+"matrix",
 "maze",
 #ifdef USE_GL
 "moebius",
