@@ -22,7 +22,7 @@ static const char sccsid[] = "@(#)color.c	4.00 97/01/01 xlockmore";
  * Create an HSB ramp.
  *
  * Revision History:
- * Changes maintained by David Bagley <bagleyd@bigfoot.com>
+ * Changes maintained by David Bagley <bagleyd@tux.org>
  * 22-Jun-94: Modified for VMS
  *            <Anthony.D.Clarke@Support.Hatfield.Raytheon.bae.eurokom.ie>
  * Changes of Patrick J. Naughton
@@ -319,8 +319,9 @@ setColormap(Display * display, Window window, Colormap cmap, Bool inwindow)
 	   un-doing any XInstallColormap() performed by a client (which is why
 	   this does not work right under Fvwm). */
 
-	if (!inwindow)
+	if (!inwindow) {
 		XInstallColormap(display, cmap);
+	}
 }
 
 /*-
