@@ -10,7 +10,7 @@ static const char sccsid[] = "@(#)iostuff.c	4.10 98/10/23 xlockmore";
  *
  * Revision History:
  *
- * Changes maintained by David Bagley <bagleyd@bigfoot.com>
+ * Changes maintained by David Bagley <bagleyd@tux.org>
  * 23-Apr-98: Separated out of util.c
  *
  */
@@ -86,7 +86,7 @@ randomFileFromList(char *directory,
 extern Bool debug;
 
 /* index_dir emulation of FORTRAN's index in C. Author: J. Jansen */
-static int
+int
 index_dir(char *str1, char *substr)
 {
 	int         i, num, l1 = strlen(str1), ls = strlen(substr), found;
