@@ -32,7 +32,8 @@ static const char sccsid[] = "@(#)mode.c	4.07 97/11/24 xlockmore";
  * they are probably not used by the mode.
  * To remove unwanted modes put a line of "#if 0" and "#endif"
  * around the mode and remove mode from the generated "modes/Makefile"
- * and/or "modes/glx/Makefile".
+ * and/or "modes/glx/Makefile".  When adding new modes, keep them in
+ * alphabetical order for -sequential to work properly.
  */
 
 #ifndef USE_MODULES
@@ -145,7 +146,7 @@ LockStruct  LockProcs[] =
 	{"dclock", init_dclock, draw_dclock, release_dclock,
 	 refresh_dclock, init_dclock, NULL, &dclock_opts,
 	 10000, 1, 10000, 1, 64, 0.3, "",
-	 "Shows a floating digital clock", 0, NULL},
+	 "Shows a floating digital clock or message", 0, NULL},
 #endif
 #ifdef MODE_decay
 	{"decay", init_decay, draw_decay, release_decay,
@@ -199,7 +200,7 @@ LockStruct  LockProcs[] =
 	{"flag", init_flag, draw_flag, release_flag,
 	 refresh_flag, init_flag, NULL, &flag_opts,
 	 50000, 1, 1000, -7, 64, 1.0, "",
-	 "Shows a waving flag of your operating system", 0, NULL},
+	 "Shows a waving flag image", 0, NULL},
 #endif
 #ifdef MODE_flame
 	{"flame", init_flame, draw_flame, release_flame,
@@ -375,6 +376,12 @@ LockStruct  LockProcs[] =
 	 100000, 1, 1, 1, 64, 1.0, "",
 	 "Shows messages", 0, NULL},
 #endif
+#ifdef MODE_matrix
+	{"matrix", init_matrix, draw_matrix, release_matrix,
+	 refresh_matrix, change_matrix, NULL, &marquee_opts,
+	 100, 1, 1, 1, 64, 1.0, "",
+	 "Shows the matrix", 0, NULL},
+#endif
 #ifdef MODE_maze
 	{"maze", init_maze, draw_maze, release_maze,
 	 refresh_maze, init_maze, NULL, &maze_opts,
@@ -494,6 +501,12 @@ LockStruct  LockProcs[] =
 	 50000, 35, 50, 1, 64, 1.0, "",
 	 "Shows slipping blits", 0, NULL},
 #endif
+#ifdef MODE_space
+	{"space", init_space, draw_space, release_space,
+	 refresh_space, init_space, NULL, &space_opts,
+	 10000, 100, 1, 15, 64, 1.0, "",
+	 "Shows a journey into deep space", 0, NULL},
+#endif
 #ifdef MODE_sphere
 	{"sphere", init_sphere, draw_sphere, release_sphere,
 	 refresh_sphere, init_sphere, NULL, &sphere_opts,
@@ -517,6 +530,8 @@ LockStruct  LockProcs[] =
 	 refresh_sproingies, init_sproingies, NULL, &sproingies_opts,
 	 1000, 5, 0, 400, 64, 1.0, "",
   "Shows Sproingies!  Nontoxic.  Safe for pets and small children", 0, NULL},
+#endif
+#ifdef MODE_stairs
 	{"stairs", init_stairs, draw_stairs, release_stairs,
 	 draw_stairs, change_stairs, NULL, &stairs_opts,
 	 200000, 0, 1, 1, 64, 1.0, "",
@@ -568,11 +583,11 @@ LockStruct  LockProcs[] =
 	{"text3d", init_text3d, draw_text3d, release_text3d,
 	 refresh_text3d, change_text3d, NULL, &text3d_opts,
 	 100000, 1, 1, 1, 64, 1.0, "",
-	 "Shows 3D texts", 0, NULL},
+	 "Shows 3D text", 0, NULL},
 #endif
 #ifdef MODE_thornbird
 	{"thornbird", init_thornbird, draw_thornbird, release_thornbird,
-	 refresh_thornbird, NULL, NULL, &thornbird_opts,
+	 refresh_thornbird, init_thornbird, NULL, &thornbird_opts,
 	 1000, 800, 16, 1, 64, 1.0, "",
 	 "Shows an animated bird in a thorn bush fractal map", 0, NULL},
 #endif
