@@ -1,9 +1,12 @@
 #ifndef __XLOCK_MODE_H__
 #define __XLOCK_MODE_H__
 
+#if !defined( lint ) && !defined( SABER )
+/* #ident  "@(#)mode.h      4.14 99/06/17 xlockmore" */
+
+#endif
+
 /*-
- * @(#)mode.h	4.13 99/02/14 xlockmore
- *
  * mode.h - mode management for xlock, the X Window System lockscreen.
  *
  * Copyright (c) 1991 by Patrick J. Naughton.
@@ -23,7 +26,7 @@
  *
  * Revision History:
  *
- * Changes maintained by David Bagley <bagleyd@bigfoot.com>
+ * Changes maintained by David Bagley <bagleyd@tux.org>
  * 18-Mar-96: Ron Hitchens <ron@idiom.com>
  *		Extensive revision to define new data types for
  *		the new mode calling scheme.
@@ -96,7 +99,6 @@
 #define MI_IS_INWINDOW(MI)	(!(MI)->root_p)
 #define MI_IS_ICONIC(MI)	(False)
 #define MI_IS_WIREFRAME(MI)	((MI)->wireframe_p)
-#define MI_IS_MOUSE(MI)	((MI)->mouse)
 #define MI_IS_USE3D(MI)	((MI)->threed)
 #define MI_COUNT(MI)	((MI)->count)
 #define MI_NCOLORS(MI)    ((MI)->ncolors)
@@ -109,6 +111,8 @@
  XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc, \
    0, 0, (unsigned int) MI_WIDTH(mi), (unsigned int) MI_HEIGHT(mi)); \
 }
+#define MI_CLEARWINDOWCOLORMAPFAST(mi, gc, pixel) \
+ MI_CLEARWINDOWCOLORMAP(mi, gc, pixel)
 #define MI_CLEARWINDOWCOLOR(mi, pixel) \
  MI_CLEARWINDOWCOLORMAP(mi, MI_GC(mi), pixel)
 
@@ -146,7 +150,6 @@ typedef struct ModeInfo {
 	long        threed_none_color;
 	long        threed_delta;
 	Bool        wireframe_p;
-	Bool        mouse;
 	char       *bitmap;
 	Bool        is_drawn;
 } ModeInfo;
@@ -266,9 +269,7 @@ typedef struct {
 	unsigned long none_pixel, both_pixel;
 	XWindowChanges fullsizeconfigure;
 	mboxInfo    mb;
-#ifdef USE_UNSTABLE
 	Pixmap	    root_pixmap;	
-#endif
 } ScreenInfo;
 
 typedef struct {
@@ -295,15 +296,14 @@ typedef struct {
 #define WI_FLAG_MONO		0x004
 #define WI_FLAG_INWINDOW	0x008
 #define WI_FLAG_INROOT		0x010
-#define WI_FLAG_NOLOCK	0x020
+#define WI_FLAG_NOLOCK		0x020
 #define WI_FLAG_INSTALL		0x040
 #define WI_FLAG_DEBUG		0x080
 #define WI_FLAG_USE3D		0x100
 #define WI_FLAG_VERBOSE		0x200
-#define WI_FLAG_FULLRANDOM		0x400
-#define WI_FLAG_WIREFRAME		0x800
-#define WI_FLAG_MOUSE		0x1000
-#define WI_FLAG_JUST_INITTED	0x2000	/* private state flag */
+#define WI_FLAG_FULLRANDOM	0x400
+#define WI_FLAG_WIREFRAME	0x800
+#define WI_FLAG_JUST_INITTED	0x1000	/* private state flag */
 
 #ifdef __cplusplus
   extern "C" {
@@ -374,7 +374,6 @@ typedef struct ModeInfo_s {
 #define MI_IS_VERBOSE(mi)	(MI_FLAG_IS_SET (mi, WI_FLAG_VERBOSE))
 #define MI_IS_FULLRANDOM(mi) (MI_FLAG_IS_SET (mi, WI_FLAG_FULLRANDOM))
 #define MI_IS_WIREFRAME(mi)	(MI_FLAG_IS_SET (mi, WI_FLAG_WIREFRAME))
-#define MI_IS_MOUSE(mi)	(MI_FLAG_IS_SET (mi, WI_FLAG_MOUSE))
 
 #define MI_SCREENINFO(mi)	((mi)->screeninfo)
 #define MI_DEPTH(mi)	((mi)->screeninfo->depth)
@@ -396,9 +395,7 @@ typedef struct ModeInfo_s {
 #define MI_NONE_COLOR(mi)	((mi)->screeninfo->none_pixel)	/* -install */
 #define MI_RIGHT_COLOR(mi)	((mi)->screeninfo->right_pixel)
 #define MI_LEFT_COLOR(mi)	((mi)->screeninfo->left_pixel)
-#ifdef USE_UNSTABLE
 #define MI_ROOT_PIXMAP(mi)	((mi)->screeninfo->root_pixmap)
-#endif
 
 #define MI_DELAY(mi)		((mi)->runinfo.delay)
 #define MI_COUNT(mi)		((mi)->runinfo.count)
@@ -439,8 +436,11 @@ extern void erase_full_window(ModeInfo * mi, GC erase_gc, unsigned long pixel);
   }
 #endif
 
-#if 0
 #define MI_CLEARWINDOWCOLORMAP(mi, gc, pixel) \
+{ \
+ erase_full_window( mi , gc , pixel ); \
+}
+#define MI_CLEARWINDOWCOLORMAPFAST(mi, gc, pixel) \
 { \
  XSetForeground(MI_DISPLAY(mi), gc, pixel); \
  if ((MI_WIDTH(mi) >= 2) || (MI_HEIGHT(mi) >= 2)) { \
@@ -452,12 +452,6 @@ extern void erase_full_window(ModeInfo * mi, GC erase_gc, unsigned long pixel);
  XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc, 0, 0, \
   (unsigned int) MI_WIDTH(mi), (unsigned int) MI_HEIGHT(mi)); \
 }
-#else
-#define MI_CLEARWINDOWCOLORMAP(mi, gc, pixel) \
-{ \
- erase_full_window( mi , gc , pixel ); \
-}
-#endif
 #define MI_CLEARWINDOWCOLOR(mi, pixel) \
  MI_CLEARWINDOWCOLORMAP(mi, MI_GC(mi), pixel)
 #if 1
@@ -512,8 +506,7 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_cartoon
 #endif
 #ifdef USE_UNSTABLE
-#define MODE_decay
-#define MODE_lyapunov
+#define MODE_space
 #endif
 #define MODE_ant
 #define MODE_ball
@@ -528,6 +521,7 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_coral
 #define MODE_crystal
 #define MODE_daisy
+#define MODE_decay
 #define MODE_dclock
 #define MODE_deco
 #define MODE_demon
@@ -560,8 +554,10 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_lisa
 #define MODE_lissie
 #define MODE_loop
+#define MODE_lyapunov
 #define MODE_mandelbrot
 #define MODE_marquee
+#define MODE_matrix
 #define MODE_maze
 #define MODE_mountain
 #define MODE_munch
@@ -1054,6 +1050,15 @@ extern ModeHook release_marquee;
 extern ModeSpecOpt marquee_opts;
 #endif
 
+#ifdef MODE_matrix
+extern ModeHook init_matrix;
+extern ModeHook draw_matrix;
+extern ModeHook release_matrix;
+extern ModeHook refresh_matrix;
+extern ModeHook change_matrix;
+extern ModeSpecOpt matrix_opts;
+#endif
+
 #ifdef MODE_maze
 extern ModeHook init_maze;
 extern ModeHook draw_maze;
@@ -1215,6 +1220,14 @@ extern ModeHook refresh_slip;	/* Probably not practical */
 extern ModeSpecOpt slip_opts;
 #endif
 
+#ifdef MODE_space
+extern ModeHook init_space;
+extern ModeHook draw_space;
+extern ModeHook release_space;
+extern ModeHook refresh_space;
+extern ModeSpecOpt space_opts;
+#endif
+
 #ifdef MODE_sphere
 extern ModeHook init_sphere;
 extern ModeHook draw_sphere;
