@@ -10,7 +10,7 @@ static const char sccsid[] = "@(#)passwd.c	4.02 97/04/01 xlockmore";
  *
  * Revision History:
  *
- * Changes maintained by David Bagley <bagleyd@bigfoot.com>
+ * Changes maintained by David Bagley <bagleyd@tux.org>
  * 18-Feb-99: allowroot option no longer ignored for PAM authentication.
  *            Sven Paas <Sven.Paas@t-online.de>
  * 24-Jan-98: Updated PAM support and made it configure-able.
@@ -50,6 +50,7 @@ extern Bool inroot;
 extern Bool inwindow;
 extern Bool grabmouse;
 extern Bool nolock;
+extern char *cpasswd;
 
 #if defined( USE_XLOCKRC ) || defined( FALLBACK_XLOCKRC )
 #include <sys/stat.h>		/* order of includes matters */
@@ -244,6 +245,9 @@ struct itmlst {
 #endif /* AFS */
 
 char        user[PASSLENGTH];
+#ifdef GLOBAL_UNLOCK
+char        global_user[PASSLENGTH];
+#endif
 
 #ifdef PAM
 #include <security/pam_appl.h>
@@ -565,7 +569,6 @@ static void
 gpass(void)
 {
 	FILE       *fp;
-	extern char *cpasswd;
 	char        xlockrc[MAXPATHLEN], *home;
 
 	if (!cpasswd || !*cpasswd) {
@@ -744,7 +747,6 @@ getCryptedUserPasswd(void)
 		(void) free((void *) buf);	/* Should never get here */
 	}
 	(void) strcpy(userpass, pw->ufld.fd_encrypt);
-
 #else /* !OSF1_ENH_SEC */
 
 	struct passwd *pw;
@@ -926,6 +928,42 @@ getCryptedRootPasswd(void)
 #endif /* !ultrix && !DCE_PASSWD && !BSD_AUTH && !PAM */
 
 
+#ifdef GLOBAL_UNLOCK
+void
+checkUser(char *buffer)
+{
+#ifdef OSF1_ENH_SEC
+	struct pr_passwd *pw;
+#else /* !OSF1_ENH_SEC */
+	struct passwd *pw;
+#ifdef HAVE_SHADOW
+	struct spwd *spw;
+
+#endif
+#endif /* !OSF1_ENH_SEC */
+	pw = (struct passwd *)getpwnam(buffer);
+
+	if (pw == NULL)
+		(void) strcpy(userpass, "*");
+	else {
+#ifdef HAVE_SHADOW
+	if ((spw = getspnam(pw->pw_name)) != NULL) {
+		char       *tmp;	/* swap */
+
+		tmp = pw->pw_passwd;
+		pw->pw_passwd = spw->sp_pwdp;
+		spw->sp_pwdp = tmp;
+	}
+	endspent();
+#endif
+		(void) strcpy(userpass, pw->pw_passwd);
+	}
+
+}
+#endif
+
+
+
 /*-
  * We do not allow for root to have no password, but we handle the case
  * where the user has no password correctly; they have to hit return
@@ -1867,8 +1905,15 @@ initPasswd(void)
 #else /* !BSD_AUTH */
 #ifdef USE_XLOCKRC
 		gpass();
+#else
+#ifdef FALLBACK_XLOCKRC
+		if (!cpasswd || !*cpasswd)
+			getCryptedUserPasswd();
+		else
+			gpass();
 #else
 		getCryptedUserPasswd();
+#endif
 #endif
 		if (allowroot)
 			getCryptedRootPasswd();
