@@ -12,7 +12,7 @@ static const char sccsid[] = "@(#)resource.c	4.08 98/08/04 xlockmore";
  *
  * Revision History:
  *
- * Changes maintained by David Bagley <bagleyd@bigfoot.com>
+ * Changes maintained by David Bagley <bagleyd@tux.org>
  * 19-Jun-98: batchcount depreciated.  Use count instead.  batchcount still
  *            works for commandline.
  * 01-May-97: Matthew Rench <mdrench@mtu.edu>
@@ -143,6 +143,7 @@ static int  descr();
 #define DEF_SATURATION	"1.0"	/* color ramp saturation 0->1 */
 #define DEF_NICE	"10"	/* xlock process nicelevel */
 #define DEF_ERASEDELAY	"400"	/* Speed for screen erase modes */
+#define DEF_ERASETIME	"2"	/* Maximum number of seconds for erase */
 #define DEF_LOCKDELAY	"0"	/* secs until lock */
 #define DEF_TIMEOUT	"30"	/* secs until password entry times out */
 #ifndef DEF_FONT
@@ -179,6 +180,9 @@ static int  descr();
 #define DEF_INFO	"Enter password to unlock; select icon to lock."
 #endif
 #endif
+#ifdef GLOBAL_UNLOCK
+#define DEF_GUSER "Username: "
+#endif
 #ifdef SAFEWORD
 #define DEF_DPASS "Dynamic password: "
 #define DEF_FPASS "Fixed Password: "
@@ -214,6 +218,7 @@ static int  descr();
 #define DEF_VTLOCK              VTLOCKMODE_OFF
 #endif
 #define DEF_CLASSNAME	"XLock"
+#if 0
 /*-
   Grid     Number of Neigbors
   ----     ------------------
@@ -223,6 +228,7 @@ static int  descr();
 */
 #define DEF_NEIGHBORS  "0"	/* automata mode will choose best or random value */
 #define DEF_MOUSE   "False"
+#endif
 
 #ifdef USE_RPLAY
 #define DEF_LOCKSOUND	"thank-you"
@@ -298,11 +304,13 @@ extern char *messagefile;
 extern char *message;		/* flag as well here */
 extern char *messagefontname;
 
+#if 0
 /* For automata modes */
 extern int  neighbors;
 
 /* For eyes, julia, & swarm modes */
 Bool        mouse;
+#endif
 
 char        hostname[MAXHOSTNAMELEN];
 static char *mode;
@@ -328,6 +336,7 @@ static XrmOptionDescRec genTable[] =
 	{"-mode", ".mode", XrmoptionSepArg, (caddr_t) NULL},
 	{"-erasemode", ".erasemode", XrmoptionSepArg, (caddr_t) NULL},
 	{"-erasedelay", ".erasedelay", XrmoptionSepArg, (caddr_t) NULL},
+	{"-erasetime", ".erasetime", XrmoptionSepArg, (caddr_t) NULL},
 	{"-nolock", ".nolock", XrmoptionNoArg, (caddr_t) "on"},
 	{"+nolock", ".nolock", XrmoptionNoArg, (caddr_t) "off"},
 #ifdef USE_VTLOCK
@@ -359,6 +368,8 @@ static XrmOptionDescRec genTable[] =
 	{"+grabmouse", ".grabmouse", XrmoptionNoArg, (caddr_t) "off"},
 	{"-grabserver", ".grabserver", XrmoptionNoArg, (caddr_t) "on"},
 	{"+grabserver", ".grabserver", XrmoptionNoArg, (caddr_t) "off"},
+	{"-hide", ".hide", XrmoptionNoArg, (caddr_t) "on"},
+	{"+hide", ".hide", XrmoptionNoArg, (caddr_t) "off"},
 	{"-install", ".install", XrmoptionNoArg, (caddr_t) "on"},
 	{"+install", ".install", XrmoptionNoArg, (caddr_t) "off"},
 	{"-mousemotion", ".mousemotion", XrmoptionNoArg, (caddr_t) "on"},
@@ -411,11 +422,13 @@ static XrmOptionDescRec genTable[] =
 	{"-messagefile", ".messagefile", XrmoptionSepArg, (caddr_t) NULL},
 	{"-message", ".message", XrmoptionSepArg, (caddr_t) NULL},
 	{"-messagefont", ".messagefont", XrmoptionSepArg, (caddr_t) NULL},
+#if 0
     /* For automata modes */
 	{"-neighbors", ".neighbors", XrmoptionSepArg, (caddr_t) NULL},
-    /* For eyes and julia modes */
+    /* For eyes, julia, and swarm modes */
 	{"-mouse", ".mouse", XrmoptionNoArg, (caddr_t) "on"},
 	{"+mouse", ".mouse", XrmoptionNoArg, (caddr_t) "off"},
+#endif
 
 #if defined( USE_XLOCKRC ) || defined( FALLBACK_XLOCKRC )
 	{"-cpasswd", ".cpasswd", XrmoptionSepArg, (caddr_t) NULL},
@@ -547,6 +560,7 @@ static OptionStruct opDesc[] =
 #endif
 	{"-erasemode erase-modename", "Erase mode to use"},
 	{"-erasedelay num", "Erase delay for clear screen modes"},
+	{"-erasetime num", "Maximum time (sec) to be used by erase"},
 	{"-/+nolock", "turn on/off no password required"},
 	{"-/+inwindow", "turn on/off making xlock run in a window"},
 	{"-/+inroot", "turn on/off making xlock run in the root window"},
@@ -570,6 +584,7 @@ static OptionStruct opDesc[] =
 	{"-/+grabmouse", "turn on/off grabbing of mouse and keyboard"},
 	{"-/+grabserver", "turn on/off grabbing of server"},
 	{"-/+install", "whether to use private colormap if needed (yes/no)"},
+	{"-/+hide", "turn on/off user background manipulation"},
 	{"-/+mousemotion", "turn on/off sensitivity to mouse"},
 	{"-/+sound", "whether to use sound if configured for it (yes/no)"},
 	{"-/+timeelapsed", "turn on/off clock"},
@@ -612,10 +627,12 @@ static OptionStruct opDesc[] =
 	{"-messagefile filename", "text file for mode"},
 	{"-message string", "text for mode"},
 	{"-messagefont fontname", "font for a specific mode"},
+#if 0
     /* For automata modes */
       {"-neighbors num", "squares 4 or 8, hexagons 6, triangles 3, 9 or 12"},
-    /* For eyes and julia modes */
+    /* For eyes, julia, and swarm modes */
 	{"-/+mouse", "turn on/off the grabbing the mouse"},
+#endif
 
 #if defined( USE_XLOCKRC ) || defined( FALLBACK_XLOCKRC )
 	{"-cpasswd crypted-password", "text string of encrypted password"},
@@ -670,7 +687,7 @@ char       *modulepath;
 
 static char *erasemodename;
 int         erasemode;
-int         erasedelay;
+int         erasedelay, erasetime;
 Bool        nolock;
 Bool        inwindow;
 Bool        inroot;
@@ -693,6 +710,7 @@ Bool        resetsaver;
 Bool        fullrandom = False;
 Bool        grabmouse;
 Bool        grabserver;
+Bool        hide;
 Bool        install;
 Bool        mousemotion;
 Bool        sound;
@@ -727,6 +745,10 @@ char       *foreground;
 char       *text_user;
 char       *text_pass;
 
+#ifdef GLOBAL_UNLOCK
+char       *text_guser;
+#endif
+
 #ifdef SAFEWORD
 char       *text_dpass;
 char       *text_fpass;
@@ -753,7 +775,7 @@ char       *right3d;
 char       *left3d;
 char       *both3d;
 
-#if defined( USE_XLOCKRC ) || defined( FALLBACK_XLOCKRC)
+#if defined( USE_XLOCKRC ) || defined( FALLBACK_XLOCKRC )
 char       *cpasswd;
 
 #endif
@@ -802,7 +824,8 @@ static argtype genvars[] =
 {
 	{(caddr_t *) & erasemodename, "erasemode", "EraseMode", "", t_String},
 	{(caddr_t *) & erasedelay, "erasedelay", "EraseDelay", DEF_ERASEDELAY, t_Int},
-    {(caddr_t *) & allowaccess, "allowaccess", "AllowAccess", "off", t_Bool},
+	{(caddr_t *) & erasetime, "erasetime", "EraseTime", DEF_ERASETIME, t_Int},
+	{(caddr_t *) & allowaccess, "allowaccess", "AllowAccess", "off", t_Bool},
 #ifndef ALWAYS_ALLOW_ROOT
 	{(caddr_t *) & allowroot, "allowroot", "AllowRoot", "off", t_Bool},
 #endif
@@ -813,6 +836,7 @@ static argtype genvars[] =
 	{(caddr_t *) & resetsaver, "resetsaver", "ResetSaver", "on", t_Bool},
 	{(caddr_t *) & grabmouse, "grabmouse", "GrabMouse", "on", t_Bool},
 	{(caddr_t *) & grabserver, "grabserver", "GrabServer", "off", t_Bool},
+	{(caddr_t *) & hide, "hide", "Hide", "on", t_Bool},
 	{(caddr_t *) & install, "install", "Install", "on", t_Bool},
     {(caddr_t *) & mousemotion, "mousemotion", "MouseMotion", "off", t_Bool},
 	{(caddr_t *) & mono, "mono", "Mono", "off", t_Bool},
@@ -833,6 +857,10 @@ static argtype genvars[] =
     {(caddr_t *) & foreground, "foreground", "Foreground", DEF_FG, t_String},
 	{(caddr_t *) & text_user, "username", "Username", DEF_NAME, t_String},
 	{(caddr_t *) & text_pass, "password", "Password", DEF_PASS, t_String},
+#ifdef GLOBAL_UNLOCK
+	{(caddr_t *) & text_guser, "globaluser", "GlobalUser", DEF_GUSER, t_String},
+#endif
+
 #ifdef SAFEWORD
 	{(caddr_t *) & text_dpass, "dynpass", "Dynpass", DEF_DPASS, t_String},
 	{(caddr_t *) & text_fpass, "fixpass", "Fixpass", DEF_FPASS, t_String},
@@ -860,8 +888,10 @@ static argtype genvars[] =
 	{(caddr_t *) & messagefile, "messagefile", "Messagefile", DEF_MESSAGEFILE, t_String},
 	{(caddr_t *) & message, "message", "Message", DEF_MESSAGE, t_String},
 	{(caddr_t *) & messagefontname, "messagefont", "MessageFont", DEF_MESSAGEFONT, t_String},
+#if 0
    {(caddr_t *) & neighbors, "neighbors", "Neighbors", DEF_NEIGHBORS, t_Int},
 	{(caddr_t *) & mouse, "mouse", "Mouse", DEF_MOUSE, t_Bool},
+#endif
 
 #if defined( USE_XLOCKRC ) || defined( FALLBACK_XLOCKRC )
 	{(caddr_t *) & cpasswd, "cpasswd", "cpasswd", "", t_String},
@@ -1046,9 +1076,9 @@ checkSpecialArgs(int argc, char **argv)
 #ifdef CHECK_OLD_ARGS
 		{
 			static char *depreciated_args[] =
-			{"-v", "-imagefile", "-mfont"};
+			{"-v", "-imagefile", "-mfont", "-rule3d", "-life3dfile", "-mouse", "-shift", "-tshift"};
 			static char *current_args[] =
-			{"-verbose", "-bitmap", "-messagefont"};
+			{"-verbose", "-bitmap", "-messagefont", "-rule", "-lifefile", "-trackmouse", "-cycle", "-cycle"};
 			int         j;
 
 			for (j = 0; j < (int) ((sizeof current_args) / sizeof (*current_args)); j++)
@@ -1681,6 +1711,13 @@ getResources(Display ** displayp, int argc, char **argv)
 	int         i, j;
 	int         max_length;
 	extern char *getenv(const char *);
+extern void XlockrmParseCommand(
+    XrmDatabase		*pdb,		/* data base */
+    register XrmOptionDescList options, /* pointer to table of valid options */
+    int			num_options,	/* number of options		     */
+    char	*prefix,	/* name to prefix resources with     */
+    int			*argc,		/* address of argument count 	     */
+    char		**argv);		/* argument list (command line)	     */
 
 #if defined( USE_AUTO_LOGOUT ) || defined( USE_BUTTON_LOGOUT )
 	extern int  fullLock();
@@ -1765,30 +1802,53 @@ getResources(Display ** displayp, int argc, char **argv)
 	XrmParseCommand(&modeDB, modeTable, modeEntries, ProgramName, &argc, argv);
 	(void) XrmMergeDatabases(modeDB, &RDB);
 
+
+
 	for (i = 0; i < numprocs; i++) {
+		/* if (!strcmp(mode, LockProcs[i].cmdline_arg)) */
 		XrmDatabase optDB = NULL;
 		ModeSpecOpt *ms = LockProcs[i].msopt;
 
 		if (!ms->numopts)
 			continue;
-		XrmParseCommand(&optDB, ms->opts, ms->numopts,
+		/* The problem was with XrmParseCommand is that it does not
+                   work for multiple use options.  Running it first with a
+                   corrupted version (that makes sure argc and argv do
+                   not change) to setup optDB then run the real 
+                   XrmParseCommand on a nullDB to set argv and argc.
+                 */
+		XlockrmParseCommand(&optDB, ms->opts, ms->numopts,
 				ProgramName, &argc, argv);
 /* PURIFY 4.0.1 on Solaris 2 reports an uninitialized memory read on the next
    * line.  PURIFY 4.0.1 on SunOS4 does not report this error. */
 		(void) XrmMergeDatabases(optDB, &RDB);
 	}
+	for (i = 0; i < numprocs; i++) {
+		/* if (!strcmp(mode, LockProcs[i].cmdline_arg)) */
+		XrmDatabase nullDB = NULL;
+		ModeSpecOpt *ms = LockProcs[i].msopt;
+
+		if (!ms->numopts)
+			continue;
+                 /*     Runnning real XrmParseCommand on a nullDB to set argv
+                        and argc.
+                 */
+		XrmParseCommand(&nullDB, ms->opts, ms->numopts,
+				ProgramName, &argc, argv);
+	}
 
 	/* the RDB is set, now query load the variables from the database */
 
-	for (i = 0; i < (int) NGENARGS; i++)
+	for (i = 0; i < (int) NGENARGS; i++) {
 		GetResource(RDB, ProgramName, classname,
 			    genvars[i].name, genvars[i].classname,
 			    genvars[i].type, genvars[i].def, genvars[i].var);
+	}
 
 #ifdef USE_VTLOCK
         /* Process the vtlock resource */
         if ( !vtlockres || !*vtlockres )
-          vtlock = False;
+        	vtlock = False;
         else {
 	    if (debug)
 		(void) fprintf(stderr,"vtlock: %s\n",vtlockres);
@@ -1871,9 +1931,11 @@ getResources(Display ** displayp, int argc, char **argv)
 		if (!ms->numvarsdesc)
 			continue;
 		v = ms->vars;
-		for (j = 0; j < ms->numvarsdesc; j++)
-			GetResource(RDB, modename, modeclassname, v[j].name, v[j].classname,
-				    v[j].type, v[j].def, v[j].var);
+		for (j = 0; j < ms->numvarsdesc; j++) {
+			GetResource(RDB, modename, modeclassname,
+				v[j].name, v[j].classname,
+				v[j].type, v[j].def, v[j].var);
+		}
 	}
 
 	/*XrmPutFileDatabase(RDB, "/tmp/xlock.rsrc.out"); */
