@@ -1,3 +1,17 @@
+#if !defined( lint ) && !defined( SABER )
+/* #ident	"@(#)vis.h	4.14 99/06/17 xlockmore" */
+
+#endif
+
+/*-
+ * Visual stuff
+ *
+ * See xlock.c for copying information.
+ *
+ * Revision History:
+ * 17-06-99: Started log. :)
+ */
+
 #ifdef __cplusplus
   extern "C" {
 #endif
