@@ -23,7 +23,13 @@ static const char sccsid[] = "@(#)xlock.c	4.11 98/05/24 xlockmore";
  *
  * Revision History:
  *
- * Changes maintained by David Bagley <bagleyd@bigfoot.com>
+ * Changes maintained by David Bagley <bagleyd@tux.org>
+ * 08-Apr-98: A way for users to unlock each others display.  Kind of defeats
+ *            the lock but the unlocked user is mailed and and entry is
+ *            written to syslogd.  Thanks to Mark Kirk <mkirk@pdi.com> for
+ *            his vizlock-1.0 patch.  Compile-time switch for this is
+ *            GLOBAL_UNLOCK.  This is probably full of security holes when
+ *            enabled... :)
  * 01-Jul-98: Eric Lassauge <lassauge@sagem.fr> &
  *              Remi Cohen-Scali <remi.cohenscali@pobox.com>
  *            Added support for locking VT switching (-/+vtlock)
@@ -204,8 +210,6 @@ extern ModeSpecOpt xlockmore_opts[];
 extern const char *app_defaults;
 char       *message, *messagefile, *messagesfile, *program;
 char       *messagefontname;
-int         neighbors;
-
 
 void
 pre_merge_options(void)
@@ -241,8 +245,8 @@ pre_merge_options(void)
 	{
 		char       *args[] =
 		{"-delay", "-count", "-cycles", "-size", "-ncolors", "-bitmap",
-		 "-text", "-filename", "-program", "-neighbors",
-		 "-wireframe", "-use3d", "-mouse", "-fullrandom", "-verbose"};
+		 "-text", "-filename", "-program",
+		 "-wireframe", "-use3d", "-verbose"};
 
 		for (j = 0; j < countof(args); j++)
 			if (strstr(app_defaults, args[j] + 1)) {
@@ -268,24 +272,6 @@ pre_merge_options(void)
 					new->specifier = options[i - 2].specifier;
 					new->argKind = XrmoptionNoArg;
 					new->value = "False";
-				} else if (!strcmp(new->option, "-mouse")) {
-					new->option = "-mouse";
-					new->argKind = XrmoptionNoArg;
-					new->value = "True";
-					new = &options[i++];
-					new->option = "-no-mouse";
-					new->specifier = options[i - 2].specifier;
-					new->argKind = XrmoptionNoArg;
-					new->value = "False";
-				} else if (!strcmp(new->option, "-fullrandom")) {
-					new->option = "-fullrandom";
-					new->argKind = XrmoptionNoArg;
-					new->value = "True";
-					new = &options[i++];
-					new->option = "-no-fullrandom";
-					new->specifier = options[i - 2].specifier;
-					new->argKind = XrmoptionNoArg;
-					new->value = "False";
 				} else if (!strcmp(new->option, "-verbose")) {
 					new->option = "-verbose";
 					new->argKind = XrmoptionNoArg;
@@ -484,7 +470,6 @@ xlockmore_screenhack(Display * dpy, Window window,
 	messagefile = get_string_resource("filename", "String");
 	messagesfile = get_string_resource("fortunefile", "String");
 	program = get_string_resource("program", "String");
-	neighbors = get_integer_resource("neighbors", "Int");
 
 
 #if 0
@@ -528,9 +513,7 @@ xlockmore_screenhack(Display * dpy, Window window,
 	mi.threed_none_color = get_pixel_resource("none3d", "Color", dpy,
 						  mi.xgwa.colormap);
 
-	mi.mouse = get_boolean_resource("mouse", "Boolean");
 	mi.wireframe_p = get_boolean_resource("wireframe", "Boolean");
-	mi.fullrandom = get_boolean_resource("fullrandom", "Boolean");
 	mi.verbose = get_boolean_resource("verbose", "Boolean");
 
 	mi.root_p = (window == RootWindowOfScreen(mi.xgwa.screen));
@@ -618,10 +601,13 @@ extern int  dpmsoff;
 
 #endif
 
-#if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
+#if ( HAVE_SYSLOG_H && (defined( USE_SYSLOG ) || defined( GLOBAL_UNLOCK )))
 #include <pwd.h>
 #include <grp.h>
 #include <syslog.h>
+#ifndef SYSLOG_FACILITY
+#define SYSLOG_FACILITY LOG_AUTH
+#endif
 #endif
 
 #if ( __VMS_VER >= 70000000 )
@@ -639,7 +625,6 @@ struct itmlst_3 {
 
 extern char *getenv(const char *);
 extern void checkResources(void);
-
 extern void defaultVisualInfo(Display * display, int screen);
 
 #ifdef USE_OLD_EVENT_LOOP
@@ -715,7 +700,6 @@ extern int  lockdelay;
 extern int  timeout;
 extern Bool wireframe;
 extern Bool use3d;
-extern Bool mouse;
 
 extern char *fontname;
 extern char *planfontname;
@@ -824,6 +808,8 @@ static int  got_invalid = 0;
 
 #endif
 
+/* This still might not be right for Solaris, change the "1"->"0" if error */
+#if 1
 #if (defined( SYSV ) || defined( SVR4 )) && defined( SOLARIS2 ) && !defined( HAVE_STRUCT_SIGSET_T )
 #if !defined( __cplusplus ) && !defined( c_plusplus ) && !defined ( _SIGSET_T )
 #define _SIGSET_T
@@ -835,6 +821,7 @@ typedef struct {		/* signal set type */
 
 #endif
 #endif
+#endif
 
 #ifdef ORIGINAL_XPM_PATCH
 #if defined( USE_XPM ) || defined( USE_XPMINC )
@@ -905,10 +892,6 @@ extern int  checkDynamic();
 #endif
 
 #if defined( HAVE_SYSLOG_H ) && defined( USE_SYSLOG )
-#ifndef SYSLOG_FACILITY
-#define SYSLOG_FACILITY LOG_AUTH
-#endif
-
 static void
 syslogStart(void)
 {
@@ -1184,7 +1167,6 @@ mode_info(Display * display, int scrn, Window window, int iconic)
 		MI_SET_FLAG_STATE(mi, WI_FLAG_VERBOSE, verbose);
 		MI_SET_FLAG_STATE(mi, WI_FLAG_FULLRANDOM, False);
 		MI_SET_FLAG_STATE(mi, WI_FLAG_WIREFRAME, wireframe);
-		MI_SET_FLAG_STATE(mi, WI_FLAG_MOUSE, mouse);
 		MI_SET_FLAG_STATE(mi, WI_FLAG_INFO_INITTED, True);
 		MI_IS_DRAWN(mi) = False;
 	}
@@ -1210,6 +1192,63 @@ mode_info(Display * display, int scrn, Window window, int iconic)
 	return (mi);
 }
 
+#ifdef GLOBAL_UNLOCK
+
+extern char *text_guser;
+extern char global_user[PASSLENGTH];
+extern void checkUser(char *buffer);
+static int unamex = 0, unamey = 0;
+#define LOG_FACILITY    LOG_LOCAL2
+#define LOG_LEVEL       LOG_INFO
+
+
+static int
+inform()
+{
+	/*
+	- the time, in date(1) style, is generated with a combination of the
+	  time(2) and ctime(3C) system calls.
+	- the owner of the xlock process is learned with getlogin(3C).
+	- global_user is a global character array  that holds the username 
+	  of the person wishing to unlock the display.
+	- the hostname is learned with gethostname(2).
+	- the syslog(3B) call uses the syslog facility to log
+	  happenings at LOG_LEVEL to LOG_FACILITY
+	- yes, a system(3S) call is used...
+	- I never said it was pretty.
+	
+	*/
+
+	char Mail[256];
+	time_t unlock_time;
+
+	/* learn what time it is and what host we are on */
+	time(&unlock_time);
+
+#if defined( HAVE_SYSLOG_H ) /* && defined( USE_SYSLOG ) */
+	/* log this event to syslogd as defined */
+	syslog(LOG_FACILITY | LOG_LEVEL, "%s: %s unlocked %s's display",
+	  ProgramName, global_user, getlogin()); 
+#endif
+	/* if (mailCmd && mailCmd[0]) { */
+		/* build a string suitable for use in system(3S) */
+		(void) sprintf(Mail, "%s -s \"%s:\" %s << EOF\n %s unlocked "
+			"%s's display on %s\n EOF",
+			/* Want the mail with the subject line control */
+#if defined( SYSV ) || defined( SVR4 ) || ( __VMS_VER >= 70000000 )
+			"/usr/ucb/mail" ,
+#else
+			"/usr/bin/mail" ,
+#endif
+			ProgramName, getlogin(),
+			global_user, hostname, ctime(&unlock_time));
+ 
+		if (debug)
+			(void) printf("%s\n", Mail);
+		(void) system(Mail); 
+	/* } */
+}
+#endif
 
 /* Restore all grabs, reset screensaver, restore colormap, close connection. */
 void
@@ -1237,6 +1276,15 @@ finish(Display * display, Bool closeDisplay)
 	if (inroot)
 		XClearWindow(display, Scr[startscreen].window);
 #endif
+
+#ifdef GLOBAL_UNLOCK
+	/* Check to see if the owner is doing the unlocking themselves,
+	and if not then inform the display owner of who is unlocking */
+
+	if (strcmp(global_user, getlogin()) != 0)
+		inform();
+#endif
+
 	if (!nolock && !allowaccess) {
 		if (grabserver)
 			XUngrabServer(display);
@@ -1731,7 +1779,7 @@ runMainLoop(int maxtime, int iconscreen)
 #endif /* !USE_OLD_EVENT_LOOP */
 
 static int
-ReadXString(char *s, int slen)
+ReadXString(char *s, int slen, Bool pass)
 {
 	XEvent      event;
 	char        keystr[20];
@@ -1818,6 +1866,16 @@ ReadXString(char *s, int slen)
 					}
 				}
 				XSetForeground(dsp, Scr[screen].gc, Scr[screen].bg_pixel);
+#ifdef GLOBAL_UNLOCK
+				if (!pass) {
+					XFillRectangle(dsp, Scr[screen].window, Scr[screen].gc,
+						 unamex, unamey - font->ascent,
+					       XTextWidth(font, s, slen),
+					       font->ascent + font->descent);
+					(void) XDrawString(dsp, Scr[screen].window, Scr[screen].textgc,
+						    unamex, unamey, s, bp);
+				} else
+#endif
 				if (echokeys) {
 					(void) memset((char *) pwbuf, '?', slen);
 					XFillRectangle(dsp, Scr[screen].window, Scr[screen].gc,
@@ -1888,12 +1946,20 @@ ReadXString(char *s, int slen)
 				if (!fullscreen)
 					break;
 				/* window config changed */
-				if (!debug && !inwindow)
+				if (!debug && !inwindow) {
 					XRaiseWindow(dsp, event.xconfigure.window);
+					fixColormap(dsp, Scr[screen].window, screen, ncolors,
+						saturation, mono, install, inroot, inwindow, verbose);
+				}
 				if (window_size_changed(screen, Scr[screen].window)) {
 					call_init_hook((LockStruct *) NULL,
 						       mode_info(dsp, screen, Scr[screen].window, False));
-				}
+				} else if (install)
+					/* next line : refresh would be logical. But some modes
+ 					 * look weird when continuing from an erased screen */
+					call_refresh_hook((LockStruct *) NULL,
+						mode_info(dsp, screen, Scr[screen].window, False));
+
 				s[0] = '\0';
 				return 1;
 			case KeymapNotify:
@@ -1954,10 +2020,6 @@ getPassword(void)
 	char       *hostbuf = NULL;
 	ModeInfo   *mi = &modeinfo[screen];
 
-#ifdef SAFEWORD
-	char        chall[PASSLENGTH];
-
-#endif
 #ifdef FX
 	Bool        mesa_3Dfx_fullscreen;
 	char       *mesa_3Dfx_env;
@@ -2019,6 +2081,15 @@ getPassword(void)
 	}
 	putText(dsp, Scr[screen].window, Scr[screen].textgc, "\n", False, left, &x, &y);
 
+#ifdef GLOBAL_UNLOCK
+	putText(dsp, Scr[screen].window, Scr[screen].textgc, text_guser,
+		True, left, &x, &y);
+	putText(dsp, Scr[screen].window, Scr[screen].textgc, " ",
+		False, left, &x, &y);
+		unamex = x;
+                unamey = y;
+	putText(dsp, Scr[screen].window, Scr[screen].textgc, "\n", False, left, &x, &y);
+#endif
 #ifdef SAFEWORD
 	if (checkDynamic()) {
 		pb = &pblock;
@@ -2194,7 +2265,16 @@ getPassword(void)
 			passy = y;
 		}
 #endif
-		if (ReadXString(buffer, PASSLENGTH))
+
+#ifdef GLOBAL_UNLOCK
+		if (ReadXString(global_user, PASSLENGTH, False))
+			break;
+		(void) XDrawString(dsp, Scr[screen].window, Scr[screen].textgc,
+			unamex, unamey,
+			global_user, strlen(global_user));
+		checkUser(global_user);
+#endif
+		if (ReadXString(buffer, PASSLENGTH, True))
 			break;
 		y = remy;
 
@@ -2377,13 +2457,22 @@ justDisplay(Display * display)
 				break;
 
 			case ConfigureNotify:
-				if (!debug && !inwindow)
+				if (!debug && !inwindow) {
 					XRaiseWindow(display, event.xconfigure.window);
+				}
 				for (screen = startscreen; screen < screens; screen++) {
+					if (install)
+						fixColormap(display, Scr[screen].window,
+							screen, ncolors, saturation,
+							mono, install, inroot, inwindow, verbose);
 					if (window_size_changed(screen, Scr[screen].window)) {
 						call_init_hook((LockStruct *) NULL,
 							       mode_info(display, screen, Scr[screen].window, False));
-					}
+					} else if (install)
+						/* next line : refresh would be logical. But some modes
+						 * look weird when continuing from an erased screen */
+						call_refresh_hook((LockStruct *) NULL,
+							mode_info(display, screen, Scr[screen].window, False));
 				}
 				break;
 
@@ -2417,7 +2506,7 @@ justDisplay(Display * display)
 		if (!nolock && lock_delay &&
 		    (lock_delay <= (int) (seconds() - start_time))) {
 			timetodie = True;
-			not_done = False;
+			/* not_done = False; */
 		}
 	}
 
@@ -2982,7 +3071,6 @@ main(int argc, char **argv)
 		Screen     *scr = ScreenOfDisplay(dsp, screen);
 		Colormap    cmap = (Colormap) NULL;
 
-#ifdef USE_UNSTABLE
 /* Start of MI_ROOT_PIXMAP hack */
 		Window 			temp_rw;
 		XGCValues		temp_gcv;
@@ -3005,7 +3093,6 @@ main(int argc, char **argv)
 		XFreeGC(dsp, temp_gc);
 						   
 /* End of MI_ROOT_PIXMAP hack */		
-#endif
 
 #ifdef ORIGINAL_XPM_PATCH
 		if (mailIcon && mailIcon[0]) {
