@@ -1,12 +1,16 @@
 #ifndef __XLOCK_XLOCK_H__
 #define __XLOCK_XLOCK_H__
 
+#if !defined( lint ) && !defined( SABER )
+/* #ident	"@(#)xlock.h	4.14 99/06/17 xlockmore" */
+
+#endif
+
 /*-
  * @(#)xlock.h	4.00 97/01/01 xlockmore 
  *
- * xlock.h - external interfaces for new modes and SYSV OS defines.
+ * External interfaces for new modes and SYSV OS defines.
  *
- * Copyright (c) 1991 by Patrick J. Naughton.
  * xscreensaver code, Copyright (c) 1997 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, and distribute this software and its
@@ -23,7 +27,7 @@
  *
  * Revision History:
  *
- * Changes maintained by David Bagley <bagleyd@bigfoot.com>
+ * Changes maintained by David Bagley <bagleyd@tux.org>
  * 23-Apr-98: Merged in xlockmore.h from xscreensaver, not much in common yet.
  * 12-May-95: Added defines for SunOS's Adjunct password file
  *            Dale A. Harris <rodmur@ecst.csuchico.edu>
@@ -218,10 +222,10 @@ XrmOptionDescRec options[100];
 #define DEF_NONE3D "Black"
 #endif
 #ifndef DEF_RIGHT3D
-#define DEF_RIGHT3D "Red"
+#define DEF_RIGHT3D "Blue"
 #endif
 #ifndef DEF_LEFT3D
-#define DEF_LEFT3D "Blue"
+#define DEF_LEFT3D "Red"
 #endif
 #ifndef DEF_BOTH3D
 #define DEF_BOTH3D "Magenta"
