@@ -19,7 +19,9 @@ static LockStruct LockProcs[] =
 {"bounce","Shows bouncing footballs"},
 {"braid","Shows random braids and knots"},
 {"bubble","Shows popping bubbles"},
+#if defined( USE_GL ) && defined( HAVE_CXX )
 {"bubble3d","Richard Jones's GL bubbles"},
+#endif
 {"bug","Shows Palmiter's bug evolution and garden of Eden"},
 #ifdef USE_GL
 {"cage","Shows the Impossible Cage, an Escher-like GL scene"},
@@ -31,7 +33,8 @@ static LockStruct LockProcs[] =
 {"coral","Shows a coral reef"},
 {"crystal","Shows polygons in 2D plane groups"},
 {"daisy","Shows a meadow of daisies"},
-{"dclock","Shows a floating digital clock"},
+{"dclock","Shows a floating digital clock or message"},
+{"decay","Shows a decaying screen"},
 {"deco","Shows art as ugly as sin"},
 {"demon","Shows Griffeath's cellular automata"},
 {"dilemma","Shows Lloyd's Prisoner's Dilemma simulation"},
@@ -39,7 +42,7 @@ static LockStruct LockProcs[] =
 {"drift","Shows cosmic drifting flame fractals"},
 {"eyes","Shows eyes following a bouncing grelb"},
 {"fadeplot","Shows a fading plot of sine squared"},
-{"flag","Shows a waving flag of your operating system"},
+{"flag","Shows a waving flag image"},
 {"flame","Shows cosmic flame fractals"},
 {"flow","Shows dynamic strange attractors"},
 {"forest","Shows binary trees of a fractal forest"},
@@ -55,9 +58,13 @@ static LockStruct LockProcs[] =
 {"ico","Shows a bouncing polyhedron"},
 {"ifs","Shows a modified iterated function system"},
 {"image","Shows randomly appearing logos"},
+{"invert","Shows a sphere inverted without wrinkles"},
 {"julia","Shows the Julia set"},
 {"kaleid","Shows a kaleidoscope"},
 {"kumppa","Shows kumppa"},
+#if defined( USE_GL ) && ( defined( USE_XPM ) || defined( USE_XPMINC ))
+{"lament","Shows Lemarchand's Box"},
+#endif
 {"laser","Shows spinning lasers"},
 {"life","Shows Conway's game of Life"},
 {"life1d","Shows Wolfram's game of 1D Life"},
@@ -66,8 +73,10 @@ static LockStruct LockProcs[] =
 {"lisa","Shows animated lisajous loops"},
 {"lissie","Shows lissajous worms"},
 {"loop","Shows Langton's self-producing loops"},
+{"lyapunov","Shows lyapunov space"},
 {"mandelbrot","Shows mandelbrot sets"},
 {"marquee","Shows messages"},
+{"matrix","Shows the matrix"},
 {"maze","Shows a random maze and a depth first search solution"},
 #ifdef USE_GL
 {"moebius","Shows Moebius Strip II, an Escher-like GL scene with ants"},
